@@ -78,6 +78,60 @@ impl Default for CipherSuite {
 }
 
 impl CipherSuite {
+    /// Returns whether the currently selected cryptographic backend
+    /// supports the encryption and signing algorithms that the cipher
+    /// suite selects.
+    pub fn is_supported(&self) -> Result<()> {
+        use crate::types::{Curve, PublicKeyAlgorithm};
+        use CipherSuite::*;
+
+        macro_rules! check_pk {
+            ($pk: expr) => {
+                if ! $pk.is_supported() {
+                    return Err(Error::UnsupportedPublicKeyAlgorithm($pk)
+                               .into());
+                }
+            }
+        }
+
+        macro_rules! check_curve {
+            ($curve: expr) => {
+                if ! $curve.is_supported() {
+                    return Err(Error::UnsupportedEllipticCurve($curve)
+                               .into());
+                }
+            }
+        }
+
+        match self {
+            Cv25519 => {
+                check_pk!(PublicKeyAlgorithm::EdDSA);
+                check_curve!(Curve::Ed25519);
+                check_pk!(PublicKeyAlgorithm::ECDH);
+                check_curve!(Curve::Cv25519);
+            },
+            RSA2k | RSA3k | RSA4k => {
+                check_pk!(PublicKeyAlgorithm::RSAEncryptSign);
+            },
+            P256 => {
+                check_pk!(PublicKeyAlgorithm::ECDSA);
+                check_curve!(Curve::NistP256);
+                check_pk!(PublicKeyAlgorithm::ECDH);
+            },
+            P384 => {
+                check_pk!(PublicKeyAlgorithm::ECDSA);
+                check_curve!(Curve::NistP384);
+                check_pk!(PublicKeyAlgorithm::ECDH);
+            },
+            P521 => {
+                check_pk!(PublicKeyAlgorithm::ECDSA);
+                check_curve!(Curve::NistP521);
+                check_pk!(PublicKeyAlgorithm::ECDH);
+            },
+        }
+        Ok(())
+    }
+
     fn generate_key<K, R>(self, flags: K)
         -> Result<Key<key::SecretParts, R>>
         where R: key::KeyRole,
@@ -149,9 +203,9 @@ assert_send_and_sync!(KeyBlueprint);
 /// certificates manually, but it should be sufficiently powerful to
 /// cover most use cases.
 ///
-/// [`UserID`s]: ../packet/struct.UserID.html
-/// [`UserAttribute`s]: ../packet/user_attribute/struct.UserAttribute.html
-/// [`Key`s]: ../packet/key/enum.Key.html
+/// [`UserID`s]: crate::packet::UserID
+/// [`UserAttribute`s]: crate::packet::user_attribute::UserAttribute
+/// [`Key`s]: crate::packet::Key
 ///
 /// # Examples
 ///
@@ -181,6 +235,7 @@ pub struct CertBuilder<'a> {
 }
 assert_send_and_sync!(CertBuilder<'_>);
 
+#[allow(clippy::new_without_default)]
 impl CertBuilder<'_> {
     /// Returns a new `CertBuilder`.
     ///
@@ -192,9 +247,9 @@ impl CertBuilder<'_> {
     /// [`CertBuilder::add_signing_subkey`],
     /// [`CertBuilder::add_transport_encryption_subkey`], etc.).
     ///
-    /// [`CertBuilder::add_signing_subkey`]: #method.add_signing_subkey
-    /// [`CertBuilder::add_transport_encryption_subkey`]: #method.add_transport_encryption_subkey
-    /// [`CertBuilder::add_userid`]: #method.add_userid
+    /// [`CertBuilder::add_signing_subkey`]: CertBuilder::add_signing_subkey()
+    /// [`CertBuilder::add_transport_encryption_subkey`]: CertBuilder::add_transport_encryption_subkey()
+    /// [`CertBuilder::add_userid`]: CertBuilder::add_userid()
     ///
     /// # Examples
     ///
@@ -303,10 +358,10 @@ impl CertBuilder<'_> {
     /// representable by a `Timestamp`.  If it is not in range,
     /// [`generate`] will return an error.
     ///
-    /// [`CertBuilder::generate`]: #method.generate
-    /// [`SystemTime`]: https://doc.rust-lang.org/stable/std/time/struct.SystemTime.html
-    /// [`Timestamp`]: ../types/struct.Timestamp.html
-    /// [`generate`]: #method.generate
+    /// [`CertBuilder::generate`]: CertBuilder::generate()
+    /// [`SystemTime`]: std::time::SystemTime
+    /// [`Timestamp`]: crate::types::Timestamp
+    /// [`generate`]: CertBuilder::generate()
     ///
     /// # Examples
     ///
@@ -392,7 +447,7 @@ impl CertBuilder<'_> {
     ///
     /// let (rsa, _) =
     ///     CertBuilder::general_purpose(None, Some("alice@example.org"))
-    ///         .set_cipher_suite(CipherSuite::RSA4k)
+    ///         .set_cipher_suite(CipherSuite::RSA2k)
     ///         .generate()?;
     /// assert_eq!(rsa.primary_key().pk_algo(), PublicKeyAlgorithm::RSAEncryptSign);
     /// # Ok(())
@@ -410,7 +465,7 @@ impl CertBuilder<'_> {
     /// [`CertBuilder::general_purpose`], will have the [primary User
     /// ID flag] set.
     ///
-    /// [`CertBuilder::general_purpose`]: #method.general_purpose
+    /// [`CertBuilder::general_purpose`]: CertBuilder::general_purpose()
     /// [primary User ID flag]: https://tools.ietf.org/html/rfc4880#section-5.2.3.19
     ///
     /// # Examples
@@ -732,8 +787,8 @@ impl CertBuilder<'_> {
     /// Use [`CertBuilder::add_subkey`] if you need to change these
     /// parameters.
     ///
-    /// [`CertBuilder::set_cipher_suite`]: #method.set_cipher_suite
-    /// [`CertBuilder::add_subkey`]: #method.add_subkey
+    /// [`CertBuilder::set_cipher_suite`]: CertBuilder::set_cipher_suite()
+    /// [`CertBuilder::add_subkey`]: CertBuilder::add_subkey()
     ///
     /// # Examples
     ///
@@ -769,8 +824,8 @@ impl CertBuilder<'_> {
     /// Use [`CertBuilder::add_subkey`] if you need to change these
     /// parameters.
     ///
-    /// [`CertBuilder::set_cipher_suite`]: #method.set_cipher_suite
-    /// [`CertBuilder::add_subkey`]: #method.add_subkey
+    /// [`CertBuilder::set_cipher_suite`]: CertBuilder::set_cipher_suite()
+    /// [`CertBuilder::add_subkey`]: CertBuilder::add_subkey()
     ///
     ///
     /// # Examples
@@ -808,8 +863,8 @@ impl CertBuilder<'_> {
     /// Use [`CertBuilder::add_subkey`] if you need to change these
     /// parameters.
     ///
-    /// [`CertBuilder::set_cipher_suite`]: #method.set_cipher_suite
-    /// [`CertBuilder::add_subkey`]: #method.add_subkey
+    /// [`CertBuilder::set_cipher_suite`]: CertBuilder::set_cipher_suite()
+    /// [`CertBuilder::add_subkey`]: CertBuilder::add_subkey()
     ///
     ///
     /// # Examples
@@ -847,8 +902,8 @@ impl CertBuilder<'_> {
     /// Use [`CertBuilder::add_subkey`] if you need to change these
     /// parameters.
     ///
-    /// [`CertBuilder::set_cipher_suite`]: #method.set_cipher_suite
-    /// [`CertBuilder::add_subkey`]: #method.add_subkey
+    /// [`CertBuilder::set_cipher_suite`]: CertBuilder::set_cipher_suite()
+    /// [`CertBuilder::add_subkey`]: CertBuilder::add_subkey()
     ///
     ///
     /// # Examples
@@ -885,8 +940,8 @@ impl CertBuilder<'_> {
     /// Use [`CertBuilder::add_subkey`] if you need to change these
     /// parameters.
     ///
-    /// [`CertBuilder::set_cipher_suite`]: #method.set_cipher_suite
-    /// [`CertBuilder::add_subkey`]: #method.add_subkey
+    /// [`CertBuilder::set_cipher_suite`]: CertBuilder::set_cipher_suite()
+    /// [`CertBuilder::add_subkey`]: CertBuilder::add_subkey()
     ///
     ///
     /// # Examples
@@ -1232,7 +1287,7 @@ impl CertBuilder<'_> {
         let creation_time =
             self.creation_time.unwrap_or_else(|| {
                 use crate::packet::signature::SIG_BACKDATE_BY;
-                time::SystemTime::now() -
+                crate::now() -
                     time::Duration::new(SIG_BACKDATE_BY, 0)
             });
 
@@ -1282,17 +1337,15 @@ impl CertBuilder<'_> {
                     a.remove_all(SubpacketTag::PrimaryUserID);
                     Ok(a)
                 })?;
+            } else if have_primary_user_thing {
+                // Check if this is the first explicitly selected
+                // user thing.
+                emitted_primary_user_thing |=
+                    sig.primary_userid().unwrap_or(false);
             } else {
-                if have_primary_user_thing {
-                    // Check if this is the first explicitly selected
-                    // user thing.
-                    emitted_primary_user_thing |=
-                        sig.primary_userid().unwrap_or(false);
-                } else {
-                    // Implicitly mark the first as primary.
-                    sig = sig.set_primary_userid(true)?;
-                    emitted_primary_user_thing = true;
-                }
+                // Implicitly mark the first as primary.
+                sig = sig.set_primary_userid(true)?;
+                emitted_primary_user_thing = true;
             }
 
             let signature = uid.bind(&mut signer, &cert, sig)?;
@@ -1314,17 +1367,15 @@ impl CertBuilder<'_> {
                     a.remove_all(SubpacketTag::PrimaryUserID);
                     Ok(a)
                 })?;
+            } else if have_primary_user_thing {
+                // Check if this is the first explicitly selected
+                // user thing.
+                emitted_primary_user_thing |=
+                    sig.primary_userid().unwrap_or(false);
             } else {
-                if have_primary_user_thing {
-                    // Check if this is the first explicitly selected
-                    // user thing.
-                    emitted_primary_user_thing |=
-                        sig.primary_userid().unwrap_or(false);
-                } else {
-                    // Implicitly mark the first as primary.
-                    sig = sig.set_primary_userid(true)?;
-                    emitted_primary_user_thing = true;
-                }
+                // Implicitly mark the first as primary.
+                sig = sig.set_primary_userid(true)?;
+                emitted_primary_user_thing = true;
             }
 
             let signature = ua.bind(&mut signer, &cert, sig)?;
@@ -1595,7 +1646,9 @@ mod tests {
     fn all_ciphersuites() {
         use self::CipherSuite::*;
 
-        for cs in vec![Cv25519, RSA3k, P256, P384, P521, RSA2k, RSA4k] {
+        for cs in vec![Cv25519, RSA3k, P256, P384, P521, RSA2k, RSA4k]
+            .into_iter().filter(|cs| cs.is_supported().is_ok())
+        {
             assert!(CertBuilder::new()
                 .set_cipher_suite(cs)
                 .generate().is_ok());
@@ -1606,7 +1659,7 @@ mod tests {
     fn validity_periods() {
         let p = &P::new();
 
-        let now = std::time::SystemTime::now();
+        let now = crate::now();
         let s = std::time::Duration::new(1, 0);
 
         let (cert,_) = CertBuilder::new()
@@ -1697,7 +1750,7 @@ mod tests {
             .generate()?;
         let cert = cert.with_policy(p, None)?;
 
-        assert_eq!(cert.revocation_keys(p).collect::<HashSet<_>>(),
+        assert_eq!(cert.revocation_keys(None).collect::<HashSet<_>>(),
                    revokers.iter().collect::<HashSet<_>>());
 
         // Do it again, with a key that has no User IDs.
@@ -1707,7 +1760,7 @@ mod tests {
         let cert = cert.with_policy(p, None)?;
         assert!(cert.primary_userid().is_err());
 
-        assert_eq!(cert.revocation_keys(p).collect::<HashSet<_>>(),
+        assert_eq!(cert.revocation_keys(None).collect::<HashSet<_>>(),
                    revokers.iter().collect::<HashSet<_>>());
 
         // The designated revokers on all signatures should be
