@@ -65,18 +65,17 @@
 //! methods that implement the [complicated semantics] for correctly
 //! locating information.
 //!
-//! [`Cert`]: ../index.html
-//! [`Packet`]: ../../packet/index.html
-//! [`Signature`]: ../../packet/signature/index.html
-//! [`ComponentBundle`]: ./struct.ComponentBundle.html
-//! [`Key`]: ../../packet/key/index.html
-//! [`UserID`]: ../../packet/struct.UserID.html
-//! [`UserAttribute`]: ../../packet/user_attribute/index.html
-//! [`Unknown`]: ../../packet/struct.Unknown.html
-//! [`Tag`]: ../../packet/enum.Tag.html
+//! [`Cert`]: super
+//! [`Packet`]: crate::packet
+//! [`Signature`]: crate::packet::signature
+//! [`Key`]: crate::packet::key
+//! [`UserID`]: crate::packet::UserID
+//! [`UserAttribute`]: crate::packet::user_attribute
+//! [`Unknown`]: crate::packet::Unknown
+//! [`Tag`]: crate::packet::Tag
 //! [designated revoker]: https://tools.ietf.org/html/rfc4880#section-5.2.3.15
-//! [`ComponentAmalgamation`]: ../amalgamation/index.html
-//! [`KeyAmalgamation`]: ../key_amalgamation/index.html
+//! [`ComponentAmalgamation`]: super::amalgamation
+//! [`KeyAmalgamation`]: super::amalgamation::key::KeyAmalgamation
 //! [complicated semantics]: https://tools.ietf.org/html/rfc4880#section-5.2.3.3
 
 use std::time;
@@ -107,7 +106,7 @@ use super::{
 
 /// A certificate component and its associated signatures.
 ///
-/// [See the module level documentation](index.html) for a detailed
+/// [See the module level documentation](self) for a detailed
 /// description.
 #[derive(Debug, Clone, PartialEq)]
 pub struct ComponentBundle<C> {
@@ -136,36 +135,36 @@ assert_send_and_sync!(ComponentBundle<C> where C);
 /// A key (primary or subkey, public or private) and any associated
 /// signatures.
 ///
-/// [See the module level documentation.](index.html)
+/// [See the module level documentation.](self)
 pub type KeyBundle<KeyPart, KeyRole> = ComponentBundle<Key<KeyPart, KeyRole>>;
 
 /// A primary key and any associated signatures.
 ///
-/// [See the module level documentation.](index.html)
+/// [See the module level documentation.](self)
 pub type PrimaryKeyBundle<KeyPart> =
     KeyBundle<KeyPart, key::PrimaryRole>;
 
 /// A subkey and any associated signatures.
 ///
-/// [See the module level documentation.](index.html)
+/// [See the module level documentation.](self)
 pub type SubkeyBundle<KeyPart>
     = KeyBundle<KeyPart, key::SubordinateRole>;
 
 /// A User ID and any associated signatures.
 ///
-/// [See the module level documentation.](index.html)
+/// [See the module level documentation.](self)
 pub type UserIDBundle = ComponentBundle<UserID>;
 
 /// A User Attribute and any associated signatures.
 ///
-/// [See the module level documentation.](index.html)
+/// [See the module level documentation.](self)
 pub type UserAttributeBundle = ComponentBundle<UserAttribute>;
 
 /// An unknown component and any associated signatures.
 ///
 /// Note: all signatures are stored as certifications.
 ///
-/// [See the module level documentation.](index.html)
+/// [See the module level documentation.](self)
 pub type UnknownBundle = ComponentBundle<Unknown>;
 
 
@@ -238,7 +237,7 @@ impl<C> ComponentBundle<C> {
                                 -> Result<&Signature>
         where T: Into<Option<time::SystemTime>>
     {
-        let t = t.into().unwrap_or_else(time::SystemTime::now);
+        let t = t.into().unwrap_or_else(crate::now);
 
         // Recall: the signatures are sorted by their creation time in
         // descending order, i.e., newest first.
@@ -497,11 +496,51 @@ impl<C> ComponentBundle<C> {
         &self.other_revocations
     }
 
+    /// Returns all of the component's Attestation Key Signatures.
+    ///
+    /// This feature is [experimental](crate#experimental-features).
+    ///
+    /// The signatures are validated, and they are sorted by their
+    /// creation time, most recent first.
+    ///
+    /// A certificate owner can use Attestation Key Signatures to
+    /// attest to third party certifications.  Currently, only userid
+    /// and user attribute certifications can be attested.  See
+    /// [Section 5.2.3.30 of RFC 4880bis] for details.
+    ///
+    ///   [Section 5.2.3.30 of RFC 4880bis]: https://tools.ietf.org/html/draft-ietf-openpgp-rfc4880bis-10.html#section-5.2.3.30
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// # use sequoia_openpgp as openpgp;
+    /// # fn main() -> openpgp::Result<()> {
+    /// # use openpgp::cert::prelude::*;
+    /// use openpgp::policy::StandardPolicy;
+    /// let p = &StandardPolicy::new();
+    ///
+    /// # let (cert, _) =
+    /// #     CertBuilder::general_purpose(None, Some("alice@example.org"))
+    /// #     .generate()?;
+    /// for (i, uid) in cert.userids().enumerate() {
+    ///     eprintln!("UserID #{} ({:?}) has {:?} attestation key signatures",
+    ///               i, uid.email(),
+    ///               uid.attestations().count());
+    /// }
+    /// # Ok(()) }
+    /// ```
+    pub fn attestations(&self)
+                      -> impl Iterator<Item = &Signature> + Send + Sync
+    {
+        self.attestations.iter()
+    }
+
     /// Returns all of the component's signatures.
     ///
     /// Only the self-signatures are validated.  The signatures are
     /// sorted first by type, then by creation time.  The self
     /// revocations come first, then the self signatures,
+    /// then any key attestation signatures,
     /// certifications, and third-party revocations coming last.  This
     /// function may return additional types of signatures that could
     /// be associated to this component.
@@ -548,6 +587,7 @@ impl<C> ComponentBundle<C> {
     ///     even if there is a newer self-signature).
     ///
     /// selfsig must be the newest live self signature at time `t`.
+    #[allow(clippy::blocks_in_if_conditions)]
     pub(crate) fn _revocation_status<'a, T>(&'a self, policy: &dyn Policy, t: T,
                                             hard_revocations_are_final: bool,
                                             selfsig: Option<&Signature>)
@@ -556,7 +596,7 @@ impl<C> ComponentBundle<C> {
     {
         // Fallback time.
         let time_zero = || time::UNIX_EPOCH;
-        let t = t.into().unwrap_or_else(time::SystemTime::now);
+        let t = t.into().unwrap_or_else(crate::now);
         let selfsig_creation_time
             = selfsig.and_then(|s| s.signature_creation_time())
                      .unwrap_or_else(time_zero);
@@ -574,10 +614,10 @@ impl<C> ComponentBundle<C> {
         let check = |revs: &'a [Signature], sec: HashAlgoSecurity|
             -> Option<Vec<&'a Signature>>
         {
-            let revs = revs.iter().filter_map(|rev| {
+            let revs = revs.iter().filter(|rev| {
                 if let Err(err) = policy.signature(rev, sec) {
                     t!("  revocation rejected by caller policy: {}", err);
-                    None
+                    false
                 } else if hard_revocations_are_final
                     && rev.reason_for_revocation()
                     .map(|(r, _)| {
@@ -593,7 +633,7 @@ impl<C> ComponentBundle<C> {
                        .unwrap_or_else(time_zero),
                        rev.reason_for_revocation()
                        .map(|r| (r.0, String::from_utf8_lossy(r.1))));
-                    Some(rev)
+                    true
                 } else if selfsig_creation_time
                     > rev.signature_creation_time().unwrap_or_else(time_zero)
                 {
@@ -602,7 +642,7 @@ impl<C> ComponentBundle<C> {
                     t!("  newer binding signature trumps soft revocation ({:?} > {:?})",
                        selfsig_creation_time,
                        rev.signature_creation_time().unwrap_or_else(time_zero));
-                    None
+                    false
                 } else if let Err(err)
                     = rev.signature_alive(t, time::Duration::new(0, 0))
                 {
@@ -613,17 +653,17 @@ impl<C> ComponentBundle<C> {
                        rev.signature_validity_period()
                            .unwrap_or_else(|| time::Duration::new(0, 0)),
                        err);
-                    None
+                    false
                 } else {
                     t!("  got a revocation: {:?} ({:?})",
                        rev.signature_creation_time().unwrap_or_else(time_zero),
                        rev.reason_for_revocation()
                            .map(|r| (r.0, String::from_utf8_lossy(r.1))));
-                    Some(rev)
+                    true
                 }
             }).collect::<Vec<&Signature>>();
 
-            if revs.len() == 0 {
+            if revs.is_empty() {
                 None
             } else {
                 Some(revs)
@@ -660,8 +700,8 @@ impl<C> ComponentBundle<C> {
     /// function uses the component's type (`C`) to determine the
     /// packet's type; the type is not a function of whether the key
     /// has secret key material.
-    pub(crate) fn into_packets<'a>(self)
-                                   -> impl Iterator<Item=Packet> + Send + Sync
+    pub(crate) fn into_packets(self)
+                               -> impl Iterator<Item=Packet> + Send + Sync
         where Packet: From<C>
     {
         let p : Packet = self.component.into();
@@ -750,7 +790,7 @@ impl<P: key::KeyParts, R: key::KeyRole> ComponentBundle<Key<P, R>> {
     /// This is just a type-specific alias for
     /// [`ComponentBundle::component`].
     ///
-    /// [`ComponentBundle::component`]: #method.component
+    /// [`ComponentBundle::component`]: ComponentBundle::component()
     ///
     /// # Examples
     ///
@@ -831,7 +871,7 @@ impl ComponentBundle<UserID> {
     /// This is just a type-specific alias for
     /// [`ComponentBundle::component`].
     ///
-    /// [`ComponentBundle::component`]: #method.component
+    /// [`ComponentBundle::component`]: ComponentBundle::component()
     ///
     /// # Examples
     ///
@@ -906,7 +946,7 @@ impl ComponentBundle<UserAttribute> {
     /// This is just a type-specific alias for
     /// [`ComponentBundle::component`].
     ///
-    /// [`ComponentBundle::component`]: #method.component
+    /// [`ComponentBundle::component`]: ComponentBundle::component()
     ///
     /// # Examples
     ///
@@ -977,7 +1017,7 @@ impl ComponentBundle<Unknown> {
     /// This is just a type-specific alias for
     /// [`ComponentBundle::component`].
     ///
-    /// [`ComponentBundle::component`]: #method.component
+    /// [`ComponentBundle::component`]: ComponentBundle::component()
     ///
     /// # Examples
     ///
