@@ -386,15 +386,15 @@ impl CertValidator {
 /// (In a [`Cert`], these show up as [`Unknown`] components.)  The
 /// goal is to provide some future compatibility.
 ///
-/// [`Packet`]: ../packet/enum.Packet.html
+/// [`Packet`]: crate::packet::Packet
 /// [`TPK`]: https://tools.ietf.org/html/rfc4880#section-11.1
 /// [`TSK`]: https://tools.ietf.org/html/rfc4880#section-11.2
-/// [`Public Key`]: ../enum.Packet.html#variant.PublicKey
-/// [`Secret Key`]: ../enum.Packet.html#variant.SecretKey
-/// [`Literal Data`]: ../enum.Packet.html#variant.Literal
-/// [`Compressed Data`]: ../enum.Packet.html#variant.CompressedData
-/// [`Cert`]: ../struct.Cert.html
-/// [`Unknown`]: ../enum.Packet.html#variant.Unknown
+/// [`Public Key`]: super::Packet::PublicKey
+/// [`Secret Key`]: super::Packet::SecretKey
+/// [`Literal Data`]: super::Packet::Literal
+/// [`Compressed Data`]: super::Packet::CompressedData
+/// [`Cert`]: super::Cert
+/// [`Unknown`]: super::Packet::Unknown
 ///
 /// # Examples
 ///
@@ -480,6 +480,7 @@ impl CertValidator {
 /// # }
 /// ```
 
+#[derive(Default)]
 pub struct CertParser<'a> {
     source: Option<Box<dyn Iterator<Item=Result<Packet>> + 'a + Send + Sync>>,
     packets: Vec<Packet>,
@@ -489,18 +490,6 @@ pub struct CertParser<'a> {
 }
 assert_send_and_sync!(CertParser<'_>);
 
-impl<'a> Default for CertParser<'a> {
-    fn default() -> Self {
-        CertParser {
-            source: None,
-            packets: vec![],
-            queued_error: None,
-            saw_error: false,
-            filter: vec![],
-        }
-    }
-}
-
 // When using a `PacketParser`, we never use the `Iter` variant.
 // Nevertheless, we need to provide a concrete type.
 // vec::IntoIter<Packet> is about as good as any other.
@@ -602,6 +591,7 @@ impl<'a> Parse<'a, CertParser<'a>> for CertParser<'a>
     }
 }
 
+#[allow(clippy::should_implement_trait)]
 impl<'a> CertParser<'a> {
     /// Creates a `CertParser` from a `Result<Packet>` iterator.
     ///
@@ -661,9 +651,10 @@ impl<'a> CertParser<'a> {
               J: 'a + Into<Result<Packet>>,
               <I as IntoIterator>::IntoIter: Send + Sync,
     {
-        let mut parser : Self = Default::default();
-        parser.source = Some(Box::new(iter.into_iter().map(Into::into)));
-        parser
+        Self {
+            source: Some(Box::new(iter.into_iter().map(Into::into))),
+            ..Default::default()
+        }
     }
 
     /// Filters the Certs prior to validation.
@@ -699,7 +690,7 @@ impl<'a> CertParser<'a> {
     /// `CertParser` will not replace the existing filter, but install
     /// multiple filters.
     ///
-    /// [`Cert`]: ../struct.Cert.html
+    /// [`Cert`]: super::Cert
     ///
     /// # Examples
     ///
@@ -782,9 +773,10 @@ impl<'a> CertParser<'a> {
     //
     // Returns the old state.  Note: the packet iterator is preserved.
     fn reset(&mut self) -> Self {
-        // We need to preserve `source`.
-        let mut orig = mem::replace(self, Default::default());
+        // We need to preserve `source` and `filter`.
+        let mut orig = mem::take(self);
         self.source = orig.source.take();
+        mem::swap(&mut self.filter, &mut orig.filter);
         orig
     }
 
@@ -903,12 +895,10 @@ pub(crate) fn split_sigs<C>(primary: &KeyHandle, primary_keyid: &KeyHandle,
             } else {
                 other_revs.push(sig);
             }
+        } else if is_selfsig {
+            self_signatures.push(sig);
         } else {
-            if is_selfsig {
-                self_signatures.push(sig);
-            } else {
-                certifications.push(sig);
-            }
+            certifications.push(sig);
         }
     }
 
@@ -1029,6 +1019,8 @@ mod test {
     use crate::serialize::Serialize;
     use crate::types::DataFormat;
 
+    use crate::tests;
+
     #[test]
     fn tokens() {
         use crate::cert::parser::low_level::lexer::{Token, Lexer};
@@ -1729,4 +1721,20 @@ mod test {
 
         Ok(())
     }
+
+    #[test]
+    fn filter() {
+        let fp = Fingerprint::from_hex(
+            "CBCD8F030588653EEDD7E2659B7DD433F254904A",
+        ).unwrap();
+
+        let cp = CertParser::from_bytes(tests::key("bad-subkey-keyring.pgp"))
+            .unwrap()
+            .unvalidated_cert_filter(|cert, _| {
+                cert.fingerprint() == fp
+            });
+        let certs = cp.collect::<Result<Vec<Cert>>>().unwrap();
+        assert_eq!(certs.len(), 1);
+        assert!(certs[0].fingerprint() == fp);
+    }
 }
