@@ -1,8 +1,8 @@
 //! Hold the implementation of [`Signer`] and [`Decryptor`] for [`KeyPair`].
 //!
-//! [`Signer`]: ../../asymmetric/trait.Signer.html
-//! [`Decryptor`]: ../../asymmetric/trait.Decryptor.html
-//! [`KeyPair`]: ../../asymmetric/struct.KeyPair.html
+//! [`Signer`]: super::super::asymmetric::Signer
+//! [`Decryptor`]: super::super::asymmetric::Decryptor
+//! [`KeyPair`]: super::super::asymmetric::KeyPair
 
 use nettle::{curve25519, ecc, ecdh, ecdsa, ed25519, dsa, rsa, random::Yarrow};
 
@@ -85,21 +85,13 @@ impl Signer for KeyPair {
                     // zeros to be stripped.
                     // Padding has to be unconditional; otherwise we have a
                     // secret-dependent branch.
-                    let missing = ed25519::ED25519_KEY_SIZE
-                        .saturating_sub(scalar.value().len());
-                    let mut sec = [0u8; ed25519::ED25519_KEY_SIZE];
-                    sec[missing..].copy_from_slice(scalar.value());
+                    let sec = scalar.value_padded(ed25519::ED25519_KEY_SIZE);
 
-                    let res = ed25519::sign(public, &sec[..], digest, &mut sig);
-                    unsafe {
-                        memsec::memzero(sec.as_mut_ptr(),
-                                        ed25519::ED25519_KEY_SIZE);
-                    }
-                    res?;
+                    ed25519::sign(public, &sec[..], digest, &mut sig)?;
 
                     Ok(mpi::Signature::EdDSA {
-                        r: MPI::new(&sig[..32]),
-                        s: MPI::new(&sig[32..]),
+                        r: MPI::new(&sig[..ed25519::ED25519_KEY_SIZE]),
+                        s: MPI::new(&sig[ed25519::ED25519_KEY_SIZE..]),
                     })
                 },
                 _ => Err(
@@ -158,25 +150,6 @@ impl Decryptor for KeyPair {
             (PublicKey::RSA{ ref e, ref n },
              mpi::SecretKeyMaterial::RSA{ ref p, ref q, ref d, .. },
              mpi::Ciphertext::RSA{ ref c }) => {
-                // Workaround for #440:  Make sure c is of the same
-                // length as n.
-                // XXX: Remove once we depend on nettle > 7.0.0.
-                let c_ = if c.value().len() < n.value().len() {
-                    let mut c_ = vec![0; n.value().len() - c.value().len()];
-                    c_.extend_from_slice(c.value());
-                    Some(c_)
-                }  else {
-                    // If it is bigger, then the packet is likely
-                    // corrupted, tough luck then.
-                    None
-                };
-                let c = if let Some(c_) = c_.as_ref() {
-                    &c_[..]
-                } else {
-                    c.value()
-                };
-                // End of workaround.
-
                 let public = rsa::PublicKey::new(n.value(), e.value())?;
                 let secret = rsa::PrivateKey::new(d.value(), p.value(),
                                                   q.value(), Option::None)?;
@@ -184,11 +157,11 @@ impl Decryptor for KeyPair {
                 if let Some(l) = plaintext_len {
                     let mut plaintext: SessionKey = vec![0; l].into();
                     rsa::decrypt_pkcs1(&public, &secret, &mut rand,
-                                       c, plaintext.as_mut())?;
+                                       c.value(), plaintext.as_mut())?;
                     plaintext
                 } else {
                     rsa::decrypt_pkcs1_insecure(&public, &secret,
-                                                &mut rand, c)?
+                                                &mut rand, c.value())?
                     .into()
                 }
             }
@@ -261,6 +234,10 @@ impl<P: key::KeyParts, R: key::KeyRole> Key<P, R> {
     {
         use crate::crypto::mpi::Signature;
 
+        fn bad(e: impl ToString) -> anyhow::Error {
+            Error::BadSignature(e.to_string()).into()
+        }
+
         let ok = match (self.mpis(), sig) {
             (PublicKey::RSA { e, n }, Signature::RSA { s }) => {
                 let key = rsa::PublicKey::new(n.value(), e.value())?;
@@ -298,26 +275,14 @@ impl<P: key::KeyParts, R: key::KeyRole> Key<P, R> {
                     // We need to zero-pad them at the front, because
                     // the MPI encoding drops leading zero bytes.
                     let half = ed25519::ED25519_SIGNATURE_SIZE / 2;
-                    if r.value().len() < half {
-                        for _ in 0..half - r.value().len() {
-                            signature.push(0);
-                        }
-                    }
-                    signature.extend_from_slice(r.value());
-                    if s.value().len() < half {
-                        for _ in 0..half - s.value().len() {
-                            signature.push(0);
-                        }
-                    }
-                    signature.extend_from_slice(s.value());
+                    signature.extend_from_slice(
+                        &r.value_padded(half).map_err(bad)?);
+                    signature.extend_from_slice(
+                        &s.value_padded(half).map_err(bad)?);
 
                     // Let's see if we got it right.
-                    if signature.len() != ed25519::ED25519_SIGNATURE_SIZE {
-                        return Err(Error::MalformedPacket(
-                            format!(
-                                "Invalid signature size: {}, r: {:?}, s: {:?}",
-                                signature.len(), r.value(), s.value())).into());
-                    }
+                    assert_eq!(signature.len(),
+                               ed25519::ED25519_SIGNATURE_SIZE);
 
                     ed25519::verify(&q.value()[1..], digest, &signature)?
                 },
@@ -379,7 +344,7 @@ impl<R> Key4<SecretParts, R>
         private_key.reverse();
 
         Self::with_secret(
-            ctime.into().unwrap_or_else(SystemTime::now),
+            ctime.into().unwrap_or_else(crate::now),
             PublicKeyAlgorithm::ECDH,
             mpi::PublicKey::ECDH {
                 curve: Curve::Cv25519,
@@ -405,7 +370,7 @@ impl<R> Key4<SecretParts, R>
         ed25519::public_key(&mut public_key, private_key).unwrap();
 
         Self::with_secret(
-            ctime.into().unwrap_or_else(SystemTime::now),
+            ctime.into().unwrap_or_else(crate::now),
             PublicKeyAlgorithm::EdDSA,
             mpi::PublicKey::EdDSA {
                 curve: Curve::Ed25519,
@@ -421,6 +386,7 @@ impl<R> Key4<SecretParts, R>
     /// The RSA key will use public exponent `e` and modulo `n`. The key will
     /// have it's creation date set to `ctime` or the current time if `None`
     /// is given.
+    #[allow(clippy::many_single_char_names)]
     pub fn import_secret_rsa<T>(d: &[u8], p: &[u8], q: &[u8], ctime: T)
         -> Result<Self> where T: Into<Option<SystemTime>>
     {
@@ -429,7 +395,7 @@ impl<R> Key4<SecretParts, R>
         let (a, b, c) = sec.as_rfc4880();
 
         Self::with_secret(
-            ctime.into().unwrap_or_else(SystemTime::now),
+            ctime.into().unwrap_or_else(crate::now),
             PublicKeyAlgorithm::RSAEncryptSign,
             mpi::PublicKey::RSA {
                 e: mpi::MPI::new(&key.e()[..]),
@@ -450,8 +416,8 @@ impl<R> Key4<SecretParts, R>
         let (public, private) = rsa::generate_keypair(&mut rng, bits as u32)?;
         let (p, q, u) = private.as_rfc4880();
         let public_mpis = PublicKey::RSA {
-            e: MPI::new(&*public.e()).into(),
-            n: MPI::new(&*public.n()).into(),
+            e: MPI::new(&*public.e()),
+            n: MPI::new(&*public.n()),
         };
         let private_mpis = mpi::SecretKeyMaterial::RSA {
             d: MPI::new(&*private.d()).into(),
@@ -461,7 +427,7 @@ impl<R> Key4<SecretParts, R>
         };
 
         Self::with_secret(
-            SystemTime::now(),
+            crate::now(),
             PublicKeyAlgorithm::RSAEncryptSign,
             public_mpis,
             private_mpis.into())
@@ -599,7 +565,7 @@ impl<R> Key4<SecretParts, R>
         };
 
         Self::with_secret(
-            SystemTime::now(),
+            crate::now(),
             pk_algo,
             mpis,
             secret)
