@@ -13,6 +13,7 @@ struct ModeWrapper<M>
     iv: Protected,
 }
 
+#[allow(clippy::new_ret_no_self)]
 impl<M> ModeWrapper<M>
 where
     M: nettle::mode::Mode + Send + Sync + 'static,
@@ -108,8 +109,9 @@ impl SymmetricAlgorithm {
         }
     }
 
-    /// Length of a key for this algorithm in bytes.  Fails if Sequoia
-    /// does not support this algorithm.
+    /// Length of a key for this algorithm in bytes.
+    ///
+    /// Fails if Sequoia does not support this algorithm.
     pub fn key_size(self) -> Result<usize> {
         match self {
             SymmetricAlgorithm::TripleDES => Ok(cipher::Des3::KEY_SIZE),
@@ -127,8 +129,9 @@ impl SymmetricAlgorithm {
         }
     }
 
-    /// Length of a block for this algorithm in bytes.  Fails if
-    /// Sequoia does not support this algorithm.
+    /// Length of a block for this algorithm in bytes.
+    ///
+    /// Fails if Sequoia does not support this algorithm.
     pub fn block_size(self) -> Result<usize> {
         match self {
             SymmetricAlgorithm::TripleDES => Ok(cipher::Des3::BLOCK_SIZE),
