@@ -40,7 +40,7 @@ pub(crate) fn encrypt_wrap<R>(recipient: &Key<key::PublicParts, R>,
     where R: key::KeyRole
 {
     match recipient.mpis() {
-        &mpi::PublicKey::ECDH { ref curve, ref hash, ref sym,.. } => {
+        mpi::PublicKey::ECDH { ref curve, ref hash, ref sym,.. } => {
             // m = sym_alg_ID || session key || checksum || pkcs5_padding;
             let mut m = Vec::with_capacity(40);
             m.extend_from_slice(session_key);
@@ -97,7 +97,7 @@ pub fn decrypt_unwrap<R>(recipient: &Key<key::PublicParts, R>,
             // Z_len = the key size for the KEK_alg_ID used with AESKeyWrap
             // Compute Z = KDF( S, Z_len, Param );
             #[allow(non_snake_case)]
-            let Z = kdf(&S, sym.key_size()?, *hash, &param)?;
+            let Z = kdf(S, sym.key_size()?, *hash, &param)?;
 
             // Compute m = AESKeyUnwrap( Z, C ) as per [RFC3394]
             let m = aes_key_unwrap(*sym, &Z, key)?;
@@ -242,7 +242,7 @@ fn aes_key_wrap(algo: SymmetricAlgorithm, key: &Protected,
             for i in 0..n {
                 // B = AES(K, A | R[i])
                 write_be_u64(&mut tmp[..8], a);
-                &mut tmp[8..].copy_from_slice(&r[8 * i..8 * (i + 1)]);
+                tmp[8..].copy_from_slice(&r[8 * i..8 * (i + 1)]);
                 cipher.encrypt(&mut b, &tmp)?;
 
                 // A = MSB(64, B) ^ t where t = (n*j)+i
@@ -251,7 +251,7 @@ fn aes_key_wrap(algo: SymmetricAlgorithm, key: &Protected,
                 // n, hence the index shift.
 
                 // R[i] = LSB(64, B)
-                &mut r[8 * i..8 * (i + 1)].copy_from_slice(&b[8..]);
+                r[8 * i..8 * (i + 1)].copy_from_slice(&b[8..]);
             }
         }
     }
@@ -317,7 +317,7 @@ fn aes_key_unwrap(algo: SymmetricAlgorithm, key: &Protected,
             for i in (0..=n-1).rev() {
                 // B = AES-1(K, (A ^ t) | R[i]) where t = n*j+i
                 write_be_u64(&mut tmp[..8], a ^ ((n * j) + i + 1) as u64);
-                &mut tmp[8..].copy_from_slice(&r[8 * i..8 * (i + 1)]);
+                tmp[8..].copy_from_slice(&r[8 * i..8 * (i + 1)]);
                 // (Note that our i runs from n-1 to 0 instead of n to
                 // 1, hence the index shift.
                 cipher.decrypt(&mut b, &tmp)?;
@@ -326,7 +326,7 @@ fn aes_key_unwrap(algo: SymmetricAlgorithm, key: &Protected,
                 a = read_be_u64(&b[..8]);
 
                 // R[i] = LSB(64, B)
-                &mut r[8 * i..8 * (i + 1)].copy_from_slice(&b[8..]);
+                r[8 * i..8 * (i + 1)].copy_from_slice(&b[8..]);
             }
         }
     }
