@@ -12,14 +12,12 @@
 //! [`Encrypted`] to protect long-term secrets like passwords and
 //! secret keys.
 //!
-//!   [`Protected`]: struct.Protected.html
-//!   [`Encrypted`]: struct.Encrypted.html
 //!
 //! Furthermore, operations involving secrets must be carried out in a
 //! way that avoids leaking information.  For example, comparison
 //! must be done in constant time with [`secure_cmp`].
 //!
-//!   [`secure_cmp`]: fn.secure_cmp.html
+//!   [`secure_cmp`]: secure_cmp()
 
 use std::cmp::{min, Ordering};
 use std::fmt;
@@ -67,14 +65,14 @@ impl Clone for Protected {
         // Make a vector with the correct size to avoid potential
         // reallocations when turning it into a `Protected`.
         let mut p = Vec::with_capacity(self.len());
-        p.extend_from_slice(&self);
+        p.extend_from_slice(self);
         p.into_boxed_slice().into()
     }
 }
 
 impl PartialEq for Protected {
     fn eq(&self, other: &Self) -> bool {
-        secure_cmp(&self, &other) == Ordering::Equal
+        secure_cmp(self, other) == Ordering::Equal
     }
 }
 
