@@ -8,21 +8,20 @@
 //! [RFC 6637] to store curves and coordinates used in elliptic curve
 //! cryptography (ECC).
 //!
-//!   [public keys]: enum.PublicKey.html
-//!   [secret keys]: enum.SecretKeyMaterial.html
-//!   [ciphertexts]: enum.Ciphertext.html
-//!   [signatures]: enum.Signature.html
+//!   [public keys]: PublicKey
+//!   [secret keys]: SecretKeyMaterial
+//!   [ciphertexts]: Ciphertext
+//!   [signatures]: Signature
 //!   [Section 3.2 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-3.2
 //!   [RFC 6637]: https://tools.ietf.org/html/rfc6637
 
 use std::fmt;
 use std::cmp::Ordering;
 use std::io::Write;
+use std::borrow::Cow;
 
 #[cfg(test)]
 use quickcheck::{Arbitrary, Gen};
-#[cfg(test)]
-use rand::Rng;
 
 use crate::types::{
     Curve,
@@ -135,6 +134,15 @@ impl MPI {
         &self.value
     }
 
+    /// Returns the value of this MPI zero-padded to the given length.
+    ///
+    /// MPI-encoding strips leading zero-bytes.  This function adds
+    /// them back, if necessary.  If the size exceeds `to`, an error
+    /// is returned.
+    pub fn value_padded(&self, to: usize) -> Result<Cow<[u8]>> {
+        crate::crypto::pad(self.value(), to)
+    }
+
     /// Decodes an EC point encoded as MPI.
     ///
     /// Decodes the MPI into a point on an elliptic curve (see
@@ -249,7 +257,7 @@ impl Hash for MPI {
 
 #[cfg(test)]
 impl Arbitrary for MPI {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
+    fn arbitrary(g: &mut Gen) -> Self {
         loop {
             let buf = <Vec<u8>>::arbitrary(g);
 
@@ -291,7 +299,6 @@ impl std::hash::Hash for MPI {
 /// The memory will be cleared when the object is dropped.  Used by
 /// [`SecretKeyMaterial`] to protect secret keys.
 ///
-///   [`SecretKeyMaterial`]: enum.SecretKeyMaterial.html
 #[derive(Clone)]
 pub struct ProtectedMPI {
     /// Integer value as big-endian.
@@ -388,6 +395,20 @@ impl ProtectedMPI {
         &self.value
     }
 
+    /// Returns the value of this MPI zero-padded to the given length.
+    ///
+    /// MPI-encoding strips leading zero-bytes.  This function adds
+    /// them back.  This operation is done unconditionally to avoid
+    /// timing differences.  If the size exceeds `to`, the result is
+    /// silently truncated to avoid timing differences.
+    pub fn value_padded(&self, to: usize) -> Protected {
+        let missing = to.saturating_sub(self.value.len());
+        let limit = self.value.len().min(to);
+        let mut v: Protected = vec![0; to].into();
+        v[missing..].copy_from_slice(&self.value()[..limit]);
+        v
+    }
+
     /// Decodes an EC point encoded as MPI.
     ///
     /// Decodes the MPI into a point on an elliptic curve (see
@@ -433,7 +454,7 @@ impl fmt::Debug for ProtectedMPI {
 /// Provides a typed and structured way of storing multiple MPIs (and
 /// the occasional elliptic curve) in [`Key`] packets.
 ///
-///   [`Key`]: ../../packet/enum.Key.html
+///   [`Key`]: crate::packet::Key
 ///
 /// Note: This enum cannot be exhaustively matched to allow future
 /// extensions.
@@ -522,13 +543,13 @@ impl PublicKey {
     pub fn bits(&self) -> Option<usize> {
         use self::PublicKey::*;
         match self {
-            &RSA { ref n,.. } => Some(n.bits()),
-            &DSA { ref p,.. } => Some(p.bits()),
-            &ElGamal { ref p,.. } => Some(p.bits()),
-            &EdDSA { ref curve,.. } => curve.bits(),
-            &ECDSA { ref curve,.. } => curve.bits(),
-            &ECDH { ref curve,.. } => curve.bits(),
-            &Unknown { .. } => None,
+            RSA { ref n,.. } => Some(n.bits()),
+            DSA { ref p,.. } => Some(p.bits()),
+            ElGamal { ref p,.. } => Some(p.bits()),
+            EdDSA { ref curve,.. } => curve.bits(),
+            ECDSA { ref curve,.. } => curve.bits(),
+            ECDH { ref curve,.. } => curve.bits(),
+            Unknown { .. } => None,
         }
     }
 
@@ -557,9 +578,11 @@ impl Hash for PublicKey {
 
 #[cfg(test)]
 impl Arbitrary for PublicKey {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
+    fn arbitrary(g: &mut Gen) -> Self {
         use self::PublicKey::*;
-        match g.gen_range(0, 6) {
+        use crate::arbitrary_helper::gen_arbitrary_from_range;
+
+        match gen_arbitrary_from_range(0..6, g) {
             0 => RSA {
                 e: MPI::arbitrary(g),
                 n: MPI::arbitrary(g),
@@ -606,8 +629,7 @@ impl Arbitrary for PublicKey {
 /// [`Key`] packets.  Secret key components are protected by storing
 /// them using [`ProtectedMPI`].
 ///
-///   [`Key`]: ../../packet/enum.Key.html
-///   [`ProtectedMPI`]: struct.ProtectedMPI.html
+///   [`Key`]: crate::packet::Key
 ///
 /// Note: This enum cannot be exhaustively matched to allow future
 /// extensions.
@@ -673,36 +695,36 @@ impl fmt::Debug for SecretKeyMaterial {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         if cfg!(debug_assertions) {
             match self {
-                &SecretKeyMaterial::RSA{ ref d, ref p, ref q, ref u } =>
+                SecretKeyMaterial::RSA{ ref d, ref p, ref q, ref u } =>
                     write!(f, "RSA {{ d: {:?}, p: {:?}, q: {:?}, u: {:?} }}", d, p, q, u),
-                &SecretKeyMaterial::DSA{ ref x } =>
+                SecretKeyMaterial::DSA{ ref x } =>
                     write!(f, "DSA {{ x: {:?} }}", x),
-                &SecretKeyMaterial::ElGamal{ ref x } =>
+                SecretKeyMaterial::ElGamal{ ref x } =>
                     write!(f, "ElGamal {{ x: {:?} }}", x),
-                &SecretKeyMaterial::EdDSA{ ref scalar } =>
+                SecretKeyMaterial::EdDSA{ ref scalar } =>
                     write!(f, "EdDSA {{ scalar: {:?} }}", scalar),
-                &SecretKeyMaterial::ECDSA{ ref scalar } =>
+                SecretKeyMaterial::ECDSA{ ref scalar } =>
                     write!(f, "ECDSA {{ scalar: {:?} }}", scalar),
-                &SecretKeyMaterial::ECDH{ ref scalar } =>
+                SecretKeyMaterial::ECDH{ ref scalar } =>
                     write!(f, "ECDH {{ scalar: {:?} }}", scalar),
-                &SecretKeyMaterial::Unknown{ ref mpis, ref rest } =>
+                SecretKeyMaterial::Unknown{ ref mpis, ref rest } =>
                     write!(f, "Unknown {{ mips: {:?}, rest: {:?} }}", mpis, rest),
             }
         } else {
             match self {
-                &SecretKeyMaterial::RSA{ .. } =>
+                SecretKeyMaterial::RSA{ .. } =>
                     f.write_str("RSA { <Redacted> }"),
-                &SecretKeyMaterial::DSA{ .. } =>
+                SecretKeyMaterial::DSA{ .. } =>
                     f.write_str("DSA { <Redacted> }"),
-                &SecretKeyMaterial::ElGamal{ .. } =>
+                SecretKeyMaterial::ElGamal{ .. } =>
                     f.write_str("ElGamal { <Redacted> }"),
-                &SecretKeyMaterial::EdDSA{ .. } =>
+                SecretKeyMaterial::EdDSA{ .. } =>
                     f.write_str("EdDSA { <Redacted> }"),
-                &SecretKeyMaterial::ECDSA{ .. } =>
+                SecretKeyMaterial::ECDSA{ .. } =>
                     f.write_str("ECDSA { <Redacted> }"),
-                &SecretKeyMaterial::ECDH{ .. } =>
+                SecretKeyMaterial::ECDH{ .. } =>
                     f.write_str("ECDH { <Redacted> }"),
-                &SecretKeyMaterial::Unknown{ .. } =>
+                SecretKeyMaterial::Unknown{ .. } =>
                     f.write_str("Unknown { <Redacted> }"),
             }
         }
@@ -715,13 +737,13 @@ impl PartialOrd for SecretKeyMaterial {
 
         fn discriminant(sk: &SecretKeyMaterial) -> usize {
             match sk {
-                &SecretKeyMaterial::RSA{ .. } => 0,
-                &SecretKeyMaterial::DSA{ .. } => 1,
-                &SecretKeyMaterial::ElGamal{ .. } => 2,
-                &SecretKeyMaterial::EdDSA{ .. } => 3,
-                &SecretKeyMaterial::ECDSA{ .. } => 4,
-                &SecretKeyMaterial::ECDH{ .. } => 5,
-                &SecretKeyMaterial::Unknown{ .. } => 6,
+                SecretKeyMaterial::RSA{ .. } => 0,
+                SecretKeyMaterial::DSA{ .. } => 1,
+                SecretKeyMaterial::ElGamal{ .. } => 2,
+                SecretKeyMaterial::EdDSA{ .. } => 3,
+                SecretKeyMaterial::ECDSA{ .. } => 4,
+                SecretKeyMaterial::ECDH{ .. } => 5,
+                SecretKeyMaterial::Unknown{ .. } => 6,
             }
         }
 
@@ -820,8 +842,10 @@ impl Hash for SecretKeyMaterial {
 
 #[cfg(test)]
 impl Arbitrary for SecretKeyMaterial {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
-        match g.gen_range(0, 6) {
+    fn arbitrary(g: &mut Gen) -> Self {
+        use crate::arbitrary_helper::gen_arbitrary_from_range;
+
+        match gen_arbitrary_from_range(0..6, g) {
             0 => SecretKeyMaterial::RSA {
                 d: MPI::arbitrary(g).into(),
                 p: MPI::arbitrary(g).into(),
@@ -881,7 +905,7 @@ impl Default for SecretKeyChecksum {
 /// Provides a typed and structured way of storing multiple MPIs in
 /// [`PKESK`] packets.
 ///
-///   [`PKESK`]: ../../packet/enum.PKESK.html
+///   [`PKESK`]: crate::packet::PKESK
 ///
 /// Note: This enum cannot be exhaustively matched to allow future
 /// extensions.
@@ -930,10 +954,10 @@ impl Ciphertext {
         // plus the big endian value itself. All other field types are
         // commented.
         match self {
-            &RSA { .. } => Some(PublicKeyAlgorithm::RSAEncryptSign),
-            &ElGamal { .. } => Some(PublicKeyAlgorithm::ElGamalEncrypt),
-            &ECDH { .. } => Some(PublicKeyAlgorithm::ECDH),
-            &Unknown { .. } => None,
+            RSA { .. } => Some(PublicKeyAlgorithm::RSAEncryptSign),
+            ElGamal { .. } => Some(PublicKeyAlgorithm::ElGamalEncrypt),
+            ECDH { .. } => Some(PublicKeyAlgorithm::ECDH),
+            Unknown { .. } => None,
         }
     }
 }
@@ -947,8 +971,10 @@ impl Hash for Ciphertext {
 
 #[cfg(test)]
 impl Arbitrary for Ciphertext {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
-        match g.gen_range(0, 3) {
+    fn arbitrary(g: &mut Gen) -> Self {
+        use crate::arbitrary_helper::gen_arbitrary_from_range;
+
+        match gen_arbitrary_from_range(0..3, g) {
             0 => Ciphertext::RSA {
                 c: MPI::arbitrary(g),
             },
@@ -976,7 +1002,7 @@ impl Arbitrary for Ciphertext {
 /// Provides a typed and structured way of storing multiple MPIs in
 /// [`Signature`] packets.
 ///
-///   [`Signature`]: ../../packet/enum.Signature.html
+///   [`Signature`]: crate::packet::Signature
 ///
 /// Note: This enum cannot be exhaustively matched to allow future
 /// extensions.
@@ -1040,8 +1066,10 @@ impl Hash for Signature {
 
 #[cfg(test)]
 impl Arbitrary for Signature {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
-        match g.gen_range(0, 4) {
+    fn arbitrary(g: &mut Gen) -> Self {
+        use crate::arbitrary_helper::gen_arbitrary_from_range;
+
+        match gen_arbitrary_from_range(0..4, g) {
             0 => Signature::RSA  {
                 s: MPI::arbitrary(g),
             },
