@@ -17,8 +17,6 @@ use std::fmt;
 
 #[cfg(test)]
 use quickcheck::{Arbitrary, Gen};
-#[cfg(test)]
-use rand::Rng;
 
 /// String-to-Key (S2K) specifiers.
 ///
@@ -29,8 +27,8 @@ use rand::Rng;
 /// [`key::Encrypted`]).
 ///
 ///   [Section 3.7 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-3.7
-///   [`SKESK`]: ../../packet/enum.SKESK.html
-///   [`key::Encrypted`]: ../../packet/key/struct.Encrypted.html
+///   [`SKESK`]: crate::packet::SKESK
+///   [`key::Encrypted`]: crate::packet::key::Encrypted
 ///
 /// Note: This enum cannot be exhaustively matched to allow future
 /// extensions.
@@ -49,7 +47,7 @@ pub enum S2K {
         /// doing a dictionary attack.  Note that not all values are
         /// representable.  See [`S2K::new_iterated`].
         ///
-        ///   [`S2K::new_iterated`]: #method.new_iterated
+        ///   [`S2K::new_iterated`]: S2K::new_iterated()
         hash_bytes: u32,
     },
 
@@ -141,7 +139,7 @@ impl S2K {
     /// parameters are chosen with contemporary machines in mind, and
     /// should also be usable on lower-end devices like smart phones.
     ///
-    ///   [`Default`]: https://doc.rust-lang.org/std/default/trait.Default.html
+    ///   [`Default`]: std::default::Default
     ///
     /// Using this method, you can tune the parameters for embedded
     /// devices.  Note, however, that this also decreases the work
@@ -198,7 +196,7 @@ impl S2K {
                             // Independent of what the hash count is, we
                             // always hash the whole salt and password once.
                             hash.update(&salt[..]);
-                            hash.update(&string);
+                            hash.update(string);
                         },
                         &S2K::Iterated { ref salt, hash_bytes, .. } => {
                             // Unroll the processing loop N times.
@@ -293,7 +291,6 @@ impl S2K {
     /// Fails with `Error::InvalidArgument` if `hash_bytes` cannot be
     /// encoded. See also [`S2K::nearest_hash_count()`].
     ///
-    /// [`S2K::nearest_hash_count()`]: #method.nearest_hash_count
     pub(crate) fn encode_count(hash_bytes: u32) -> Result<u8> {
         // eeee.mmmm -> (16 + mmmm) * 2^(6 + e)
 
@@ -305,7 +302,7 @@ impl S2K {
                             hash_bytes)).into());
             }
             11..=32 => {
-                let m = 0b1111_000000 << (msb - 11);
+                let m = 0b11_1100_0000 << (msb - 11);
                 let t = 1 << (msb - 11);
 
                 (m, t - 1)
@@ -367,21 +364,23 @@ impl fmt::Display for S2K {
 
 #[cfg(test)]
 impl Arbitrary for S2K {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
+    fn arbitrary(g: &mut Gen) -> Self {
+        use crate::arbitrary_helper::gen_arbitrary_from_range;
+
         #[allow(deprecated)]
-        match g.gen_range(0, 7) {
+        match gen_arbitrary_from_range(0..7, g) {
             0 => S2K::Simple{ hash: HashAlgorithm::arbitrary(g) },
             1 => S2K::Salted{
                 hash: HashAlgorithm::arbitrary(g),
-                salt: g.gen(),
+                salt: [<u8>::arbitrary(g); 8],
             },
             2 => S2K::Iterated{
                 hash: HashAlgorithm::arbitrary(g),
-                salt: g.gen(),
-                hash_bytes: S2K::nearest_hash_count(g.gen()),
+                salt: [<u8>::arbitrary(g); 8],
+                hash_bytes: S2K::nearest_hash_count(Arbitrary::arbitrary(g)),
             },
             3 => S2K::Private {
-                tag: g.gen_range(100, 111),
+                tag: gen_arbitrary_from_range(100..111, g),
                 parameters: Option::<Vec<u8>>::arbitrary(g).map(|v| v.into()),
             },
             4 => S2K::Unknown {
@@ -389,11 +388,11 @@ impl Arbitrary for S2K {
                 parameters: Option::<Vec<u8>>::arbitrary(g).map(|v| v.into()),
             },
             5 => S2K::Unknown {
-                tag: g.gen_range(4, 100),
+                tag: gen_arbitrary_from_range(4..100, g),
                 parameters: Option::<Vec<u8>>::arbitrary(g).map(|v| v.into()),
             },
             6 => S2K::Unknown {
-                tag: g.gen_range(111, 256) as u8,
+                tag: gen_arbitrary_from_range(111..256, g) as u8,
                 parameters: Option::<Vec<u8>>::arbitrary(g).map(|v| v.into()),
             },
             _ => unreachable!(),
@@ -553,7 +552,7 @@ mod tests {
                 S2K::Unknown { tag, .. } =>
                     (tag > 3 && tag < 100) || tag == 2 || tag > 110,
                 S2K::Private { tag, .. } =>
-                    tag >= 100 && tag <= 110,
+                    (100..=110).contains(&tag),
                 _ => true
             }
         }
