@@ -23,10 +23,10 @@ use quickcheck::{Arbitrary, Gen};
 ///
 ///   [RFC 4880]: https://tools.ietf.org/html/rfc4880
 ///   [Section 12.2 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-12.2
-///   [`KeyID`]: ./enum.KeyID.html
-///   [`KeyHandle`]: ./enum.KeyHandle.html
-///   [`IssuerFingerprint`]: ./packet/signature/subpacket/enum.SubpacketValue.html#variant.IssuerFingerprint
+///   [`IssuerFingerprint`]: crate::packet::signature::subpacket::SubpacketValue::IssuerFingerprint
 ///   [birthday attack]: https://nullprogram.com/blog/2019/07/22/
+///   [`KeyID`]: crate::KeyID
+///   [`KeyHandle`]: crate::KeyHandle
 ///
 /// Note: This enum cannot be exhaustively matched to allow future
 /// extensions.
@@ -141,8 +141,8 @@ impl Fingerprint {
     /// ```
     pub fn as_bytes(&self) -> &[u8] {
         match self {
-            &Fingerprint::V4(ref fp) => fp,
-            &Fingerprint::Invalid(ref fp) => fp,
+            Fingerprint::V4(ref fp) => fp,
+            Fingerprint::Invalid(ref fp) => fp,
         }
     }
 
@@ -187,7 +187,7 @@ impl Fingerprint {
     ///
     /// See also [`Fingerprint::to_icao`].
     ///
-    ///   [`Fingerprint::to_icao`]: #method.to_icao
+    ///   [`Fingerprint::to_icao`]: Fingerprint::to_icao()
     ///
     /// ```rust
     /// # fn main() -> sequoia_openpgp::Result<()> {
@@ -233,8 +233,8 @@ impl Fingerprint {
     /// Common code for the above functions.
     fn convert_to_string(&self, pretty: bool) -> String {
         let raw = match self {
-            &Fingerprint::V4(ref fp) => &fp[..],
-            &Fingerprint::Invalid(ref fp) => &fp[..],
+            Fingerprint::V4(ref fp) => &fp[..],
+            Fingerprint::Invalid(ref fp) => &fp[..],
         };
 
         // We currently only handle V4 fingerprints, which look like:
@@ -345,10 +345,9 @@ impl Fingerprint {
 
 #[cfg(test)]
 impl Arbitrary for Fingerprint {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
-        use rand::Rng;
+    fn arbitrary(g: &mut Gen) -> Self {
         let mut fp = [0; 20];
-        fp.iter_mut().for_each(|p| *p = g.gen());
+        fp.iter_mut().for_each(|p| *p = Arbitrary::arbitrary(g));
         Fingerprint::V4(fp)
     }
 }
