@@ -36,10 +36,10 @@ use crate::Result;
 ///
 ///   [RFC 4880]: https://tools.ietf.org/html/rfc4880
 ///   [Section 12.2 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-12.2
-///   [`Fingerprint`]: ./enum.Fingerprint.html
-///   [`KeyHandle`]: ./enum.KeyHandle.html
 ///   [birthday attack]: https://nullprogram.com/blog/2019/07/22/
-///   [`Issuer`]: ./packet/signature/subpacket/enum.SubpacketValue.html#variant.Issuer
+///   [`Issuer`]: crate::packet::signature::subpacket::SubpacketValue::Issuer
+///   [`Fingerprint`]: crate::Fingerprint
+///   [`KeyHandle`]: crate::KeyHandle
 ///
 /// Note: This enum cannot be exhaustively matched to allow future
 /// extensions.
@@ -240,8 +240,8 @@ impl KeyID {
     /// ```
     pub fn as_bytes(&self) -> &[u8] {
         match self {
-            &KeyID::V4(ref id) => id,
-            &KeyID::Invalid(ref id) => id,
+            KeyID::V4(ref id) => id,
+            KeyID::Invalid(ref id) => id,
         }
     }
 
@@ -353,8 +353,8 @@ impl KeyID {
     /// Common code for the above functions.
     fn convert_to_string(&self, pretty: bool) -> String {
         let raw = match self {
-            &KeyID::V4(ref fp) => &fp[..],
-            &KeyID::Invalid(ref fp) => &fp[..],
+            KeyID::V4(ref fp) => &fp[..],
+            KeyID::Invalid(ref fp) => &fp[..],
         };
 
         // We currently only handle V4 Key IDs, which look like:
@@ -400,7 +400,7 @@ impl KeyID {
 
 #[cfg(test)]
 impl Arbitrary for KeyID {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
+    fn arbitrary(g: &mut Gen) -> Self {
         KeyID::new(u64::arbitrary(g))
     }
 }
