@@ -88,11 +88,7 @@ impl MessageValidity {
     /// Note: a `MessageValidator` will only return this after
     /// `MessageValidator::finish` has been called.
     pub fn is_message(&self) -> bool {
-        if let MessageValidity::Message = self {
-            true
-        } else {
-            false
-        }
+        matches!(self, MessageValidity::Message)
     }
 
     /// Returns whether the packet sequence forms a valid message
@@ -101,21 +97,13 @@ impl MessageValidity {
     /// Note: a `MessageValidator` will only return this before
     /// `MessageValidator::finish` has been called.
     pub fn is_message_prefix(&self) -> bool {
-        if let MessageValidity::MessagePrefix = self {
-            true
-        } else {
-            false
-        }
+        matches!(self, MessageValidity::MessagePrefix)
     }
 
     /// Returns whether the packet sequence is definitely not a valid
     /// OpenPGP Message.
     pub fn is_err(&self) -> bool {
-        if let MessageValidity::Error(_) = self {
-            true
-        } else {
-            false
-        }
+        matches!(self, MessageValidity::Error(_))
     }
 }
 
@@ -176,7 +164,7 @@ impl MessageValidator {
         assert!(!self.finished);
         assert!(self.depth.is_some());
         assert!(token != Token::Pop);
-        assert!(path.len() > 0);
+        assert!(!path.is_empty());
 
         if self.error.is_some() {
             return;
@@ -229,7 +217,7 @@ impl MessageValidator {
                     Error::MalformedMessage(
                         format!("Invalid OpenPGP message: \
                                  {:?} packet (at {:?}) not expected",
-                                tag, path).into())));
+                                tag, path))));
                 self.tokens.clear();
                 return;
             }
@@ -356,7 +344,7 @@ impl<'a> Parse<'a, Message> for Message {
     ///
     /// See [`Message::try_from`] for more details.
     ///
-    ///   [`Message::try_from`]: #method.try_from
+    ///   [`Message::try_from`]: Message::try_from()
     fn from_reader<R: 'a + io::Read + Send + Sync>(reader: R) -> Result<Self> {
         Self::try_from(PacketPile::from_reader(reader)?)
     }
@@ -365,7 +353,7 @@ impl<'a> Parse<'a, Message> for Message {
     ///
     /// See [`Message::try_from`] for more details.
     ///
-    ///   [`Message::try_from`]: #method.try_from
+    ///   [`Message::try_from`]: Message::try_from()
     fn from_file<P: AsRef<Path>>(path: P) -> Result<Self> {
         Self::try_from(PacketPile::from_file(path)?)
     }
@@ -374,7 +362,7 @@ impl<'a> Parse<'a, Message> for Message {
     ///
     /// See [`Message::try_from`] for more details.
     ///
-    ///   [`Message::try_from`]: #method.try_from
+    ///   [`Message::try_from`]: Message::try_from()
     fn from_bytes<D: AsRef<[u8]> + ?Sized + Send + Sync>(data: &'a D) -> Result<Self> {
         Self::try_from(PacketPile::from_bytes(data)?)
     }
@@ -420,7 +408,7 @@ impl Message {
     /// ```
     pub fn body(&self) -> Option<&Literal> {
         for packet in self.pile.descendants() {
-            if let &Packet::Literal(ref l) = packet {
+            if let Packet::Literal(ref l) = packet {
                 return Some(l);
             }
         }
@@ -454,7 +442,7 @@ impl TryFrom<PacketPile> for Message {
                         Error::MalformedMessage(
                             format!("Invalid OpenPGP message: \
                                      {:?} packet (at {:?}) not expected: {}",
-                                    u.tag(), path, u.error()).into()))
+                                    u.tag(), path, u.error())))
                                .into()),
                 _ => v.push(packet.tag(), &path),
             }
@@ -481,7 +469,7 @@ impl TryFrom<PacketPile> for Message {
             // We really want to squash the lexer's error: it is an
             // internal detail that may change, and meaningless even
             // to an immediate user of this crate.
-            MessageValidity::Error(e) => Err(e.into()),
+            MessageValidity::Error(e) => Err(e),
         }
     }
 }
@@ -493,7 +481,7 @@ impl TryFrom<Vec<Packet>> for Message {
     ///
     /// See [`Message::try_from`] for more details.
     ///
-    ///   [`Message::try_from`]: #method.try_from
+    ///   [`Message::try_from`]: Message::try_from()
     fn try_from(packets: Vec<Packet>) -> Result<Self> {
         Self::try_from(PacketPile::from(packets))
     }
@@ -811,6 +799,7 @@ mod tests {
         assert!(message.is_ok(), "{:?}", message);
     }
 
+    #[allow(clippy::vec_init_then_push)]
     #[test]
     fn compressed_part() {
         let mut lit = Literal::new(Text);
@@ -873,6 +862,7 @@ mod tests {
         assert!(message.is_ok(), "{:?}", message);
     }
 
+    #[allow(clippy::vec_init_then_push)]
     #[test]
     fn one_pass_sig_part() {
         let mut lit = Literal::new(Text);
@@ -985,6 +975,7 @@ mod tests {
         assert!(message.is_ok(), "{:?}", message);
     }
 
+    #[allow(clippy::vec_init_then_push)]
     #[test]
     fn signature_part() {
         let mut lit = Literal::new(Text);
@@ -1194,7 +1185,7 @@ mod tests {
         let mut l = MessageValidator::new();
         l.push_token(Token::Literal, &[0]);
         l.finish();
-        assert!(if let MessageValidity::Message = l.check() { true } else { false });
+        assert!(matches!(l.check(), MessageValidity::Message));
     }
 
     #[test]
@@ -1205,7 +1196,7 @@ mod tests {
         l.push_token(Token::Literal, &[0]);
         l.finish();
 
-        assert!(if let MessageValidity::Message = l.check() { true } else { false });
+        assert!(matches!(l.check(), MessageValidity::Message));
     }
 
     #[test]
@@ -1217,7 +1208,7 @@ mod tests {
         l.push(Tag::Literal, &[0]);
         l.finish();
 
-        assert!(if let MessageValidity::Message = l.check() { true } else { false });
+        assert!(matches!(l.check(), MessageValidity::Message));
     }
 
     #[test]
@@ -1227,7 +1218,7 @@ mod tests {
         let mut l = MessageValidator::new();
         l.finish();
 
-        assert!(if let MessageValidity::Error(_) = l.check() { true } else { false });
+        assert!(matches!(l.check(), MessageValidity::Error(_)));
     }
 
     #[test]
@@ -1237,17 +1228,17 @@ mod tests {
 
         // No packets will return an error.
         let mut l = MessageValidator::new();
-        assert!(if let MessageValidity::MessagePrefix = l.check() { true } else { false });
+        assert!(matches!(l.check(), MessageValidity::MessagePrefix));
         l.finish();
 
-        assert!(if let MessageValidity::Error(_) = l.check() { true } else { false });
+        assert!(matches!(l.check(), MessageValidity::Error(_)));
 
         // Simple one-literal message.
         let mut l = MessageValidator::new();
         l.push(Tag::Literal, &[0]);
-        assert!(if let MessageValidity::MessagePrefix = l.check() { true } else { false });
+        assert!(matches!(l.check(), MessageValidity::MessagePrefix));
         l.finish();
 
-        assert!(if let MessageValidity::Message = l.check() { true } else { false });
+        assert!(matches!(l.check(), MessageValidity::Message));
     }
 }
