@@ -8,11 +8,11 @@ use std::hash::{Hash, Hasher};
 use std::slice;
 use std::vec;
 
+use xxhash_rust::xxh3::Xxh3;
+
 use crate::{
     Packet,
-    crypto::hash,
     packet::Iter,
-    types::HashAlgorithm,
 };
 
 /// A packet's body holds either unprocessed bytes, processed bytes,
@@ -28,25 +28,25 @@ use crate::{
 /// a particular packet using the
 /// [`PacketParser::buffer_unread_content`] method.
 ///
-///   [`PacketParserBuilder::buffer_unread_content`]: ../parse/struct.PacketParserBuilder.html#method.buffer_unread_content
-///   [`PacketPile`]: ../struct.PacketPile.html
-///   [`PacketParser::buffer_unread_content`]: ../parse/struct.PacketParser.html#method.buffer_unread_content
+///   [`PacketParserBuilder::buffer_unread_content`]: crate::parse::PacketParserBuilder::buffer_unread_content()
+///   [`PacketPile`]: crate::PacketPile
+///   [`PacketParser::buffer_unread_content`]: crate::parse::PacketParser::buffer_unread_content()
 ///
 /// There are three different types of packets:
 ///
-///   - Most packets like the [`UserID`] and [`Signature`] packets, don't
+///   - Most packets, like the [`UserID`] and [`Signature`] packets, don't
 ///     actually have a body.
 ///
-///   [`UserID`]: ../packet/struct.UserID.html
-///   [`Signature`]: ../packet/signature/struct.Signature.html
+///   [`UserID`]: crate::packet::UserID
+///   [`Signature`]: crate::packet::Signature
 ///
 ///   - Some packets have an unprocessed body.  The [`Literal`] data
 ///     packet wraps unstructured plaintext, and the [`Unknown`]
 ///     packet contains data that we failed to process, say because we
 ///     didn't support the packet's version.
 ///
-///   [`Literal`]: ../packet/struct.Literal.html
-///   [`Unknown`]: ../packet/struct.Unknown.html
+///   [`Literal`]: crate::packet::Literal
+///   [`Unknown`]: crate::packet::Unknown
 ///
 ///   - Some packets are containers.  If the parser does not parse the
 ///     packet's child, either because the caller used
@@ -57,7 +57,7 @@ use crate::{
 ///     deserialization routines, since the content is just an OpenPGP
 ///     message.)
 ///
-///   [`PacketParser::next`]: ../parse/struct.PacketParser.html#method.next
+///   [`PacketParser::next`]: crate::parse::PacketParser::next()
 #[derive(Clone, Debug)]
 pub enum Body {
     /// Unprocessed packet body.
@@ -79,10 +79,10 @@ pub enum Body {
     /// this is not the packet's entire content; it is just the unread
     /// content.
     ///
-    ///   [`Literal`]: ../packet/struct.Literal.html
-    ///   [`Unknown`]: ../packet/struct.Unknown.html
-    ///   [`SEIP`]: ../packet/enum.SEIP.html
-    ///   [`AED`]: ../packet/enum.AED.html
+    ///   [`Literal`]: crate::packet::Literal
+    ///   [`Unknown`]: crate::packet::Unknown
+    ///   [`SEIP`]: crate::packet::SEIP
+    ///   [`AED`]: crate::packet::AED
     Unprocessed(Vec<u8>),
 
     /// Processed packed body.
@@ -108,9 +108,9 @@ pub enum Body {
     /// [`PacketParser`] deserialization routines; this needs to be
     /// done manually.
     ///
-    ///   [`PacketPile`]: ../struct.PacketPile.html
-    ///   [`PacketPile::from_file`]: ../struct.PacketPile.html#method.from_file
-    ///   [`PacketParser`]: ../parse/struct.PacketParser.html
+    ///   [`PacketPile`]: crate::PacketPile
+    ///   [`PacketPile::from_file`]: crate::PacketPile#method.from_file
+    ///   [`PacketParser`]: crate::parse::PacketParser
     Structured(Vec<Packet>),
 }
 
@@ -126,7 +126,7 @@ pub struct Container {
     body: Body,
 
     /// We compute a digest over the body to implement comparison.
-    body_digest: Vec<u8>,
+    body_digest: u64,
 }
 
 assert_send_and_sync!(Container);
@@ -138,14 +138,6 @@ impl std::ops::Deref for Container {
     }
 }
 
-// Pick the fastest hash function from the SHA2 family for the
-// architectures word size.  On 64-bit architectures, SHA512 is almost
-// twice as fast, but on 32-bit ones, SHA256 is faster.
-#[cfg(target_pointer_width = "64")]
-const CONTAINER_BODY_HASH: HashAlgorithm = HashAlgorithm::SHA512;
-#[cfg(not(target_pointer_width = "64"))]
-const CONTAINER_BODY_HASH: HashAlgorithm = HashAlgorithm::SHA256;
-
 impl PartialEq for Container {
     fn eq(&self, other: &Container) -> bool {
         use Body::*;
@@ -177,7 +169,7 @@ impl Default for Container {
     fn default() -> Self {
         Self {
             body: Body::Structured(Vec::with_capacity(0)),
-            body_digest: Vec::with_capacity(0),
+            body_digest: 0,
         }
     }
 }
@@ -186,7 +178,7 @@ impl From<Vec<Packet>> for Container {
     fn from(packets: Vec<Packet>) -> Self {
         Self {
             body: Body::Structured(packets),
-            body_digest: Vec::with_capacity(0),
+            body_digest: 0,
         }
     }
 }
@@ -270,7 +262,7 @@ impl Container {
     /// Returns an iterator over the packet's immediate children.
     ///
     /// Returns `None` if the body is not structured.
-    pub fn children<'a>(&'a self) -> Option<slice::Iter<'a, Packet>> {
+    pub fn children(&self) -> Option<slice::Iter<Packet>> {
         Some(self.children_ref()?.iter())
     }
 
@@ -304,36 +296,31 @@ impl Container {
     }
 
     /// Returns the hash for the empty body.
-    fn empty_body_digest() -> Vec<u8> {
+    fn empty_body_digest() -> u64 {
         lazy_static::lazy_static!{
-            static ref DIGEST: Vec<u8> = {
-                let mut h = Container::make_body_hash();
-                let mut d = vec![0; h.digest_size()];
-                let _ = h.digest(&mut d);
-                d
+            static ref DIGEST: u64 = {
+                Container::make_body_hash().digest()
             };
         }
 
-        DIGEST.clone()
+        *DIGEST
     }
 
     /// Creates a hash context for hashing the body.
     pub(crate) // For parse.rs
-    fn make_body_hash() -> Box<dyn hash::Digest> {
-        CONTAINER_BODY_HASH.context()
-            .expect("CONTAINER_BODY_HASH must be implemented")
+    fn make_body_hash() -> Box<Xxh3> {
+        Box::new(Xxh3::new())
     }
 
     /// Hashes content that has been streamed.
     pub(crate) // For parse.rs
-    fn set_body_hash(&mut self, mut h: Box<dyn hash::Digest>) {
-        self.body_digest.resize(h.digest_size(), 0);
-        let _ = h.digest(&mut self.body_digest);
+    fn set_body_hash(&mut self, h: Box<Xxh3>) {
+        self.body_digest = h.digest();
     }
 
     pub(crate)
     fn body_digest(&self) -> String {
-        crate::fmt::hex::encode(&self.body_digest)
+        format!("{:08X}", self.body_digest)
     }
 
     // Converts an indentation level to whitespace.
@@ -353,7 +340,7 @@ impl Container {
         for (i, p) in self.children_ref().iter().enumerate() {
             eprintln!("{}{}: {:?}",
                       Self::indent(indent), i + 1, p);
-            if let Some(ref children) = self.children_ref()
+            if let Some(children) = self.children_ref()
                 .and_then(|c| c.get(i)).and_then(|p| p.container_ref())
             {
                 children.pretty_print(indent + 1);
@@ -435,8 +422,8 @@ impl Packet {
     }
 
     /// Returns an iterator over the packet's immediate children.
-    pub(crate) fn children<'a>(&'a self)
-                               -> Option<impl Iterator<Item = &'a Packet>> {
+    pub(crate) fn children(& self)
+                           -> Option<impl Iterator<Item = &Packet>> {
         self.container_ref().and_then(|c| c.children())
     }
 
