@@ -130,33 +130,27 @@
 //!
 //! [packet based]: https://tools.ietf.org/html/rfc4880#section-5
 //! [the grammar]: https://tools.ietf.org/html/rfc4880#section-11
-//! [`Signature`]: enum.Signature.html
 //! [v3]: https://tools.ietf.org/html/rfc4880#section-5.2.2
 //! [v4]: https://tools.ietf.org/html/rfc4880#section-5.2.3
-//! [`OnePassSig`]: enum.OnePassSig.html
-//! [parsing a message]: ../parse/index.html
-//! [creating a message]: ../serialize/stream/index.html
-//! [`SignatureBuilder`]: signature/struct.SignatureBuilder.html
+//! [parsing a message]: crate::parse
+//! [creating a message]: crate::serialize::stream
+//! [`SignatureBuilder`]: signature::SignatureBuilder
 //! [`SED`]: https://tools.ietf.org/html/rfc4880#section-5.7
-//! [`Unknown`]: struct.Unknown.html
 //! [private extensions]: https://tools.ietf.org/html/rfc4880#section-4.3
-//! [`Compressed Data`]: struct.CompressedData.html
-//! [parses]: ../parse/index.html
+//! [`Compressed Data`]: CompressedData
+//! [parses]: crate::parse
 //! [OpenPGP Message]: https://tools.ietf.org/html/rfc4880#section-11.3
-//! [`Container::descendants`]: struct.Container.html#method.descendants
-//! [`Deref`]: https://doc.rust-lang.org/stable/std/ops/trait.Deref.html
-//! [`Container`]: struct.Container.html
-//! [`Signature`]: enum.Signature.html
-//! [`SubpacketArea`]: signature/subpacket/struct.SubpacketArea.html
-//! [`Literal`]: struct.Literal.html
-//! [`Body`]: enum.Body.html
-//! [`Eq`]: https://doc.rust-lang.org/stable/std/cmp/trait.Eq.html
-//! [`Key`s]: enum.Key.html
-//! [`CTB`]: header/enum.CTB.html
+//! [`Container::descendants`]: Container::descendants()
+//! [`Deref`]: std::ops::Deref
+//! [`SubpacketArea`]: signature::subpacket::SubpacketArea
+//! [`Eq`]: std::cmp::Eq
+//! [`Key`s]: Key
+//! [`CTB`]: header::CTB
 //! [length style]: https://tools.ietf.org/html/rfc4880#section-4.2
 //! [partial body encoding]: https://tools.ietf.org/html/rfc4880#section-4.2.2.4
-//! [`Key::public_cmp`]: enum.Key.html#method.public_cmp
+//! [`Key::public_cmp`]: Key::public_cmp()
 use std::fmt;
+use std::hash::Hasher;
 use std::ops::{Deref, DerefMut};
 use std::slice;
 use std::iter::IntoIterator;
@@ -216,6 +210,8 @@ pub mod aed;
 ///
 /// The different OpenPGP packets are detailed in [Section 5 of RFC 4880].
 ///
+///   [Section 5 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-5
+///
 /// The [`Unknown`] packet allows Sequoia to deal with packets that it
 /// doesn't understand.  It is basically a binary blob that includes
 /// the packet's [tag].  See the [module-level documentation] for
@@ -232,10 +228,10 @@ pub mod aed;
 /// are equal, modulo the OpenPGP framing ([`CTB`] and [length style],
 /// potential [partial body encoding]).
 ///
-/// [`Unknown`]: struct.Unknown.html
+/// [`Unknown`]: crate::packet::Unknown
 /// [tag]: https://tools.ietf.org/html/rfc4880#section-4.3
-/// [module-level documentation]: index.html#unknown-packets
-/// [`CTB`]: header/enum.CTB.html
+/// [module-level documentation]: crate::packet#unknown-packets
+/// [`CTB`]: crate::packet::header::CTB
 /// [length style]: https://tools.ietf.org/html/rfc4880#section-4.2
 /// [partial body encoding]: https://tools.ietf.org/html/rfc4880#section-4.2.2.4
 #[non_exhaustive]
@@ -334,24 +330,24 @@ impl Packet {
     ///   [Section 4.3 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-4.3
     pub fn tag(&self) -> Tag {
         match self {
-            &Packet::Unknown(ref packet) => packet.tag(),
-            &Packet::Signature(_) => Tag::Signature,
-            &Packet::OnePassSig(_) => Tag::OnePassSig,
-            &Packet::PublicKey(_) => Tag::PublicKey,
-            &Packet::PublicSubkey(_) => Tag::PublicSubkey,
-            &Packet::SecretKey(_) => Tag::SecretKey,
-            &Packet::SecretSubkey(_) => Tag::SecretSubkey,
-            &Packet::Marker(_) => Tag::Marker,
-            &Packet::Trust(_) => Tag::Trust,
-            &Packet::UserID(_) => Tag::UserID,
-            &Packet::UserAttribute(_) => Tag::UserAttribute,
-            &Packet::Literal(_) => Tag::Literal,
-            &Packet::CompressedData(_) => Tag::CompressedData,
-            &Packet::PKESK(_) => Tag::PKESK,
-            &Packet::SKESK(_) => Tag::SKESK,
-            &Packet::SEIP(_) => Tag::SEIP,
-            &Packet::MDC(_) => Tag::MDC,
-            &Packet::AED(_) => Tag::AED,
+            Packet::Unknown(ref packet) => packet.tag(),
+            Packet::Signature(_) => Tag::Signature,
+            Packet::OnePassSig(_) => Tag::OnePassSig,
+            Packet::PublicKey(_) => Tag::PublicKey,
+            Packet::PublicSubkey(_) => Tag::PublicSubkey,
+            Packet::SecretKey(_) => Tag::SecretKey,
+            Packet::SecretSubkey(_) => Tag::SecretSubkey,
+            Packet::Marker(_) => Tag::Marker,
+            Packet::Trust(_) => Tag::Trust,
+            Packet::UserID(_) => Tag::UserID,
+            Packet::UserAttribute(_) => Tag::UserAttribute,
+            Packet::Literal(_) => Tag::Literal,
+            Packet::CompressedData(_) => Tag::CompressedData,
+            Packet::PKESK(_) => Tag::PKESK,
+            Packet::SKESK(_) => Tag::SKESK,
+            Packet::SEIP(_) => Tag::SEIP,
+            Packet::MDC(_) => Tag::MDC,
+            Packet::AED(_) => Tag::AED,
         }
     }
 
@@ -364,63 +360,104 @@ impl Packet {
     /// whereas `kind()` returns `None`.
     pub fn kind(&self) -> Option<Tag> {
         match self {
-            &Packet::Unknown(_) => None,
+            Packet::Unknown(_) => None,
             _ => Some(self.tag()),
         }
     }
+
+    /// Hashes most everything into state.
+    ///
+    /// This is an alternate implementation of [`Hash`], which does
+    /// not hash:
+    ///
+    ///   - The unhashed subpacket area of Signature packets.
+    ///   - Secret key material.
+    ///
+    ///   [`Hash`]: std::hash::Hash
+    ///
+    /// Unlike [`Signature::normalize`], this method ignores
+    /// authenticated packets in the unhashed subpacket area.
+    ///
+    ///   [`Signature::normalize`]: Signature::normalize()
+    pub fn normalized_hash<H>(&self, state: &mut H)
+        where H: Hasher
+    {
+        use std::hash::Hash;
+
+        match self {
+            Packet::Signature(sig) => sig.normalized_hash(state),
+            Packet::OnePassSig(x) => Hash::hash(&x, state),
+            Packet::PublicKey(k) => k.public_hash(state),
+            Packet::PublicSubkey(k) => k.public_hash(state),
+            Packet::SecretKey(k) => k.public_hash(state),
+            Packet::SecretSubkey(k) => k.public_hash(state),
+            Packet::Marker(x) => Hash::hash(&x, state),
+            Packet::Trust(x) => Hash::hash(&x, state),
+            Packet::UserID(x) => Hash::hash(&x, state),
+            Packet::UserAttribute(x) => Hash::hash(&x, state),
+            Packet::Literal(x) => Hash::hash(&x, state),
+            Packet::CompressedData(x) => Hash::hash(&x, state),
+            Packet::PKESK(x) => Hash::hash(&x, state),
+            Packet::SKESK(x) => Hash::hash(&x, state),
+            Packet::SEIP(x) => Hash::hash(&x, state),
+            Packet::MDC(x) => Hash::hash(&x, state),
+            Packet::AED(x) => Hash::hash(&x, state),
+            Packet::Unknown(x) => Hash::hash(&x, state),
+        }
+    }
 }
 
 // Allow transparent access of common fields.
-impl<'a> Deref for Packet {
+impl Deref for Packet {
     type Target = Common;
 
     fn deref(&self) -> &Self::Target {
         match self {
-            &Packet::Unknown(ref packet) => &packet.common,
-            &Packet::Signature(ref packet) => &packet.common,
-            &Packet::OnePassSig(ref packet) => &packet.common,
-            &Packet::PublicKey(ref packet) => &packet.common,
-            &Packet::PublicSubkey(ref packet) => &packet.common,
-            &Packet::SecretKey(ref packet) => &packet.common,
-            &Packet::SecretSubkey(ref packet) => &packet.common,
-            &Packet::Marker(ref packet) => &packet.common,
-            &Packet::Trust(ref packet) => &packet.common,
-            &Packet::UserID(ref packet) => &packet.common,
-            &Packet::UserAttribute(ref packet) => &packet.common,
-            &Packet::Literal(ref packet) => &packet.common,
-            &Packet::CompressedData(ref packet) => &packet.common,
-            &Packet::PKESK(ref packet) => &packet.common,
-            &Packet::SKESK(SKESK::V4(ref packet)) => &packet.common,
-            &Packet::SKESK(SKESK::V5(ref packet)) => &packet.skesk4.common,
-            &Packet::SEIP(ref packet) => &packet.common,
-            &Packet::MDC(ref packet) => &packet.common,
-            &Packet::AED(ref packet) => &packet.common,
+            Packet::Unknown(ref packet) => &packet.common,
+            Packet::Signature(ref packet) => &packet.common,
+            Packet::OnePassSig(ref packet) => &packet.common,
+            Packet::PublicKey(ref packet) => &packet.common,
+            Packet::PublicSubkey(ref packet) => &packet.common,
+            Packet::SecretKey(ref packet) => &packet.common,
+            Packet::SecretSubkey(ref packet) => &packet.common,
+            Packet::Marker(ref packet) => &packet.common,
+            Packet::Trust(ref packet) => &packet.common,
+            Packet::UserID(ref packet) => &packet.common,
+            Packet::UserAttribute(ref packet) => &packet.common,
+            Packet::Literal(ref packet) => &packet.common,
+            Packet::CompressedData(ref packet) => &packet.common,
+            Packet::PKESK(ref packet) => &packet.common,
+            Packet::SKESK(SKESK::V4(ref packet)) => &packet.common,
+            Packet::SKESK(SKESK::V5(ref packet)) => &packet.skesk4.common,
+            Packet::SEIP(ref packet) => &packet.common,
+            Packet::MDC(ref packet) => &packet.common,
+            Packet::AED(ref packet) => &packet.common,
         }
     }
 }
 
-impl<'a> DerefMut for Packet {
+impl DerefMut for Packet {
     fn deref_mut(&mut self) -> &mut Common {
         match self {
-            &mut Packet::Unknown(ref mut packet) => &mut packet.common,
-            &mut Packet::Signature(ref mut packet) => &mut packet.common,
-            &mut Packet::OnePassSig(ref mut packet) => &mut packet.common,
-            &mut Packet::PublicKey(ref mut packet) => &mut packet.common,
-            &mut Packet::PublicSubkey(ref mut packet) => &mut packet.common,
-            &mut Packet::SecretKey(ref mut packet) => &mut packet.common,
-            &mut Packet::SecretSubkey(ref mut packet) => &mut packet.common,
-            &mut Packet::Marker(ref mut packet) => &mut packet.common,
-            &mut Packet::Trust(ref mut packet) => &mut packet.common,
-            &mut Packet::UserID(ref mut packet) => &mut packet.common,
-            &mut Packet::UserAttribute(ref mut packet) => &mut packet.common,
-            &mut Packet::Literal(ref mut packet) => &mut packet.common,
-            &mut Packet::CompressedData(ref mut packet) => &mut packet.common,
-            &mut Packet::PKESK(ref mut packet) => &mut packet.common,
-            &mut Packet::SKESK(SKESK::V4(ref mut packet)) => &mut packet.common,
-            &mut Packet::SKESK(SKESK::V5(ref mut packet)) => &mut packet.skesk4.common,
-            &mut Packet::SEIP(ref mut packet) => &mut packet.common,
-            &mut Packet::MDC(ref mut packet) => &mut packet.common,
-            &mut Packet::AED(ref mut packet) => &mut packet.common,
+            Packet::Unknown(ref mut packet) => &mut packet.common,
+            Packet::Signature(ref mut packet) => &mut packet.common,
+            Packet::OnePassSig(ref mut packet) => &mut packet.common,
+            Packet::PublicKey(ref mut packet) => &mut packet.common,
+            Packet::PublicSubkey(ref mut packet) => &mut packet.common,
+            Packet::SecretKey(ref mut packet) => &mut packet.common,
+            Packet::SecretSubkey(ref mut packet) => &mut packet.common,
+            Packet::Marker(ref mut packet) => &mut packet.common,
+            Packet::Trust(ref mut packet) => &mut packet.common,
+            Packet::UserID(ref mut packet) => &mut packet.common,
+            Packet::UserAttribute(ref mut packet) => &mut packet.common,
+            Packet::Literal(ref mut packet) => &mut packet.common,
+            Packet::CompressedData(ref mut packet) => &mut packet.common,
+            Packet::PKESK(ref mut packet) => &mut packet.common,
+            Packet::SKESK(SKESK::V4(ref mut packet)) => &mut packet.common,
+            Packet::SKESK(SKESK::V5(ref mut packet)) => &mut packet.skesk4.common,
+            Packet::SEIP(ref mut packet) => &mut packet.common,
+            Packet::MDC(ref mut packet) => &mut packet.common,
+            Packet::AED(ref mut packet) => &mut packet.common,
         }
     }
 }
@@ -471,9 +508,10 @@ impl fmt::Debug for Packet {
 
 #[cfg(test)]
 impl Arbitrary for Packet {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
-        use rand::Rng;
-        match g.gen_range(0, 14) {
+    fn arbitrary(g: &mut Gen) -> Self {
+        use crate::arbitrary_helper::gen_arbitrary_from_range;
+
+        match gen_arbitrary_from_range(0..15, g) {
             0 => Signature::arbitrary(g).into(),
             1 => OnePassSig::arbitrary(g).into(),
             2 => Key::<key::PublicParts, key::UnspecifiedRole>::arbitrary(g)
@@ -492,13 +530,31 @@ impl Arbitrary for Packet {
             11 => CompressedData::arbitrary(g).into(),
             12 => PKESK::arbitrary(g).into(),
             13 => SKESK::arbitrary(g).into(),
+            14 => loop {
+                let mut u = Unknown::new(
+                    Tag::arbitrary(g), anyhow::anyhow!("Arbitrary::arbitrary"));
+                u.set_body(Arbitrary::arbitrary(g));
+                let u = Packet::Unknown(u);
+
+                // Check that we didn't accidentally make a valid
+                // packet.
+                use crate::parse::Parse;
+                use crate::serialize::SerializeInto;
+                if let Ok(Packet::Unknown(_)) = Packet::from_bytes(
+                    &u.to_vec().unwrap())
+                {
+                    break u;
+                }
+
+                // Try again!
+            },
             _ => unreachable!(),
         }
     }
 }
 
 /// Fields used by multiple packet types.
-#[derive(Debug, Clone)]
+#[derive(Default, Debug, Clone)]
 pub struct Common {
     // In the future, this structure will hold the parsed CTB, packet
     // length, and lengths of chunks of partial body encoded packets.
@@ -518,20 +574,12 @@ assert_send_and_sync!(Common);
 
 #[cfg(test)]
 impl Arbitrary for Common {
-    fn arbitrary<G: Gen>(_: &mut G) -> Self {
+    fn arbitrary(_: &mut Gen) -> Self {
         // XXX: Change if this gets interesting fields.
         Common::default()
     }
 }
 
-impl Default for Common {
-    fn default() -> Common {
-        Common {
-            dummy: Default::default(),
-        }
-    }
-}
-
 impl PartialEq for Common {
     fn eq(&self, _: &Common) -> bool {
         // Don't compare anything.
@@ -573,13 +621,9 @@ impl std::hash::Hash for Common {
 /// This is returned by [`PacketPile::descendants`] and
 /// [`Container::descendants`].
 ///
-/// [`Packet`]: enum.Packet.html
-/// [container `Packet`s]: index.html#containers
-/// [`CompressedData`]: struct.CompressedData.html
-/// [`SEIP`]: enum.SEIP.html
-/// [`AED`]: enum.AED.html
-/// [`PacketPile::descendants`]: ../struct.PacketPile.html#method.descendants
-/// [`Container::descendants`]: struct.Container.html#method.descendants
+/// [container `Packet`s]: self#containers
+/// [`PacketPile::descendants`]: super::PacketPile::descendants()
+/// [`Container::descendants`]: Container::descendants()
 pub struct Iter<'a> {
     // An iterator over the current message's children.
     children: slice::Iter<'a, Packet>,
@@ -626,13 +670,13 @@ impl<'a> Iterator for Iter<'a> {
         // Get the next child and the iterator for its children.
         self.child = self.children.next();
         if let Some(child) = self.child {
-            self.grandchildren = child.descendants().map(|d| Box::new(d));
+            self.grandchildren = child.descendants().map(Box::new);
         }
 
         // First return the child itself.  Subsequent calls will
         // return its grandchildren.
         self.depth = 0;
-        return self.child;
+        self.child
     }
 }
 
@@ -646,7 +690,7 @@ impl<'a> Iter<'a> {
     /// See [`PacketPile::path_ref`] for an explanation of
     /// `pathspec`s.
     ///
-    /// [`PacketPile::path_ref`]: ../struct.PacketPile.html#path_ref
+    /// [`PacketPile::path_ref`]: super::PacketPile::path_ref
     ///
     /// # Examples
     ///
@@ -717,8 +761,7 @@ impl<'a> Iter<'a> {
 ///
 /// Like [`Iter::enumerate`].
 ///
-/// [`Iter::enumerate`]: https://doc.rust-lang.org/stable/std/iter/trait.Iterator.html#method.enumerate
-/// [`Iter`]: struct.Iter.html
+/// [`Iter::enumerate`]: std::iter::Iterator::enumerate()
 struct PacketPathIter<'a> {
     iter: Iter<'a>,
 
@@ -775,7 +818,7 @@ fn packet_path_iter() {
             v.push(i);
             lpaths.push(v);
 
-            if let Some(ref container) = packet.container_ref() {
+            if let Some(container) = packet.container_ref() {
                 if let Some(c) = container.children() {
                     for mut path in paths(c).into_iter()
                     {
@@ -842,14 +885,14 @@ fn packet_path_iter() {
 /// the [`SignatureBuilder`] to create the `Signature` packet.  See
 /// the linked documentation for details, and examples.
 ///
-/// [streaming `Signer`]: ../serialize/stream/struct.Signer.html
-/// [`PacketParser`]: ../parse/index.html
-/// [self signatures on Keys]: enum.Key.html#method.bind
-/// [self signatures on User IDs]: struct.UserID.html#method.bind
-/// [self signatures on User Attributes]: user_attribute/struct.UserAttribute.html#method.bind
-/// [certifications of User IDs]: struct.UserID.html#method.certify
-/// [certifications of User Attributes]: user_attribute/struct.UserAttribute.html#method.certify
-/// [`SignatureBuilder`]: signature/struct.SignatureBuilder.html
+/// [streaming `Signer`]: crate::serialize::stream::Signer
+/// [`PacketParser`]: crate::parse::PacketParser
+/// [self signatures on Keys]: Key::bind()
+/// [self signatures on User IDs]: UserID::bind()
+/// [self signatures on User Attributes]: user_attribute::UserAttribute::bind()
+/// [certifications of User IDs]: UserID::certify()
+/// [certifications of User Attributes]: user_attribute::UserAttribute::certify()
+/// [`SignatureBuilder`]: signature::SignatureBuilder
 ///
 /// Note: This enum cannot be exhaustively matched to allow future
 /// extensions.
@@ -868,7 +911,7 @@ fn packet_path_iter() {
 /// service vector, if `Signature`s are naively deduplicated.  To
 /// protect against this, consider using [`Signature::normalized_eq`].
 ///
-///   [`Signature::normalized_eq`]: #method.normalized_eq
+///   [`Signature::normalized_eq`]: Signature::normalized_eq()
 ///
 /// # Examples
 ///
@@ -922,7 +965,7 @@ impl Signature {
     /// Gets the version.
     pub fn version(&self) -> u8 {
         match self {
-            &Signature::V4(_) => 4,
+            Signature::V4(_) => 4,
         }
     }
 }
@@ -966,8 +1009,8 @@ impl DerefMut for Signature {
 /// extensions.
 ///
 /// [Section 5.4 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-5.4
-/// [`PacketParser`]: ../parse/index.html
-/// [streaming serializer]: ../serialize/stream/index.html
+/// [`PacketParser`]: crate::parse::PacketParser
+/// [streaming serializer]: crate::serialize::stream
 #[non_exhaustive]
 #[derive(PartialEq, Eq, Hash, Clone, Debug)]
 pub enum OnePassSig {
@@ -980,7 +1023,7 @@ impl OnePassSig {
     /// Gets the version.
     pub fn version(&self) -> u8 {
         match self {
-            &OnePassSig::V3(_) => 3,
+            OnePassSig::V3(_) => 3,
         }
     }
 }
@@ -1025,11 +1068,9 @@ impl DerefMut for OnePassSig {
 /// Note: This enum cannot be exhaustively matched to allow future
 /// extensions.
 ///
-/// [SEIP]: enum.SEIP.html
-/// [AED]: enum.AED.html
 /// [Section 5.1 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-5.1
-/// [streaming serializer]: ../serialize/stream/index.html
-/// [`PacketParser`]: ../parse/index.html
+/// [streaming serializer]: crate::serialize::stream
+/// [`PacketParser`]: crate::parse::PacketParser
 #[non_exhaustive]
 #[derive(PartialEq, Eq, Hash, Clone, Debug)]
 pub enum PKESK {
@@ -1087,11 +1128,9 @@ impl DerefMut for PKESK {
 /// Note: This enum cannot be exhaustively matched to allow future
 /// extensions.
 ///
-/// [SEIP]: enum.SEIP.html
-/// [AED]: enum.AED.html
 /// [Section 5.3 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-5.3
-/// [streaming serializer]: ../serialize/stream/index.html
-/// [`PacketParser`]: ../parse/index.html
+/// [streaming serializer]: crate::serialize::stream
+/// [`PacketParser`]: crate::parse::PacketParser
 #[non_exhaustive]
 #[derive(PartialEq, Eq, Hash, Clone, Debug)]
 pub enum SKESK {
@@ -1099,7 +1138,7 @@ pub enum SKESK {
     V4(self::skesk::SKESK4),
     /// SKESK packet version 5.
     ///
-    /// This feature is [experimental](../index.html#experimental-features).
+    /// This feature is [experimental](super#experimental-features).
     V5(self::skesk::SKESK5),
 }
 assert_send_and_sync!(SKESK);
@@ -1108,8 +1147,8 @@ impl SKESK {
     /// Gets the version.
     pub fn version(&self) -> u8 {
         match self {
-            &SKESK::V4(_) => 4,
-            &SKESK::V5(_) => 5,
+            SKESK::V4(_) => 4,
+            SKESK::V5(_) => 5,
         }
     }
 }
@@ -1179,13 +1218,7 @@ impl From<SKESK> for Packet {
 /// [secret keys]: https://tools.ietf.org/html/rfc4880#section-5.5.1.3
 /// [public subkeys]: https://tools.ietf.org/html/rfc4880#section-5.5.1.2
 /// [secret subkeys]: https://tools.ietf.org/html/rfc4880#section-5.5.1.4
-/// [`key::PublicParts`]: key/struct.PublicParts.html
-/// [`key::SecretParts`]: key/struct.SecretParts.html
-/// [`key::UnspecifiedParts`]: key/struct.UnspecifiedParts.html
-/// [`key::PrimaryRole`]: key/struct.PrimaryRole.html
-/// [`key::SubordinateRole`]: key/struct.SubordinateRole.html
-/// [`key::UnspecifiedRole`]: key/struct.UnspecifiedRole.html
-/// [`Cert::keys`]: ../struct.Cert.html#method.keys
+/// [`Cert::keys`]: super::Cert::keys()
 ///
 /// ## Examples
 ///
@@ -1248,20 +1281,20 @@ impl From<SKESK> for Packet {
 ///
 /// It is also possible to use `From`.
 ///
-/// [`Signer`]: ../crypto/trait.Signer.html
-/// [`Key::parts_as_secret`]: enum.Key.html#method.parts_as_secret
-/// [`Key::parts_into_public`]: #method.parts_into_public
-/// [`Key::parts_as_public`]: #method.parts_as_public
-/// [`Key::parts_into_secret`]: #method.parts_into_secret
-/// [`Key::parts_as_secret`]: #method.parts_as_secret
-/// [`Key::parts_into_unspecified`]: #method.parts_into_unspecified
-/// [`Key::parts_as_unspecified`]: #method.parts_as_unspecified
-/// [`Key::role_into_primary`]: #method.role_into_primary
-/// [`Key::role_as_primary`]: #method.role_as_primary
-/// [`Key::role_into_subordinate`]: #method.role_into_subordinate
-/// [`Key::role_as_subordinate`]: #method.role_as_subordinate
-/// [`Key::role_into_unspecified`]: #method.role_into_unspecified
-/// [`Key::role_as_unspecified`]: #method.role_as_unspecified
+/// [`Signer`]: super::crypto::Signer
+/// [`Key::parts_as_secret`]: Key::parts_as_secret()
+/// [`Key::parts_into_public`]: Key::parts_into_public()
+/// [`Key::parts_as_public`]: Key::parts_as_public()
+/// [`Key::parts_into_secret`]: Key::parts_into_secret()
+/// [`Key::parts_as_secret`]: Key::parts_as_secret()
+/// [`Key::parts_into_unspecified`]: Key::parts_into_unspecified()
+/// [`Key::parts_as_unspecified`]: Key::parts_as_unspecified()
+/// [`Key::role_into_primary`]: Key::role_into_primary()
+/// [`Key::role_as_primary`]: Key::role_as_primary()
+/// [`Key::role_into_subordinate`]: Key::role_into_subordinate()
+/// [`Key::role_as_subordinate`]: Key::role_as_subordinate()
+/// [`Key::role_into_unspecified`]: Key::role_into_unspecified()
+/// [`Key::role_as_unspecified`]: Key::role_as_unspecified()
 ///
 /// ## Examples
 ///
@@ -1301,7 +1334,7 @@ impl From<SKESK> for Packet {
 /// converted to a secret key.  This is necessary, for instance, when
 /// creating a [`Signer`]:
 ///
-/// [`Cert`]: ../struct.Cert.html
+/// [`Cert`]: super::Cert
 ///
 /// ```rust
 /// use std::time;
@@ -1348,8 +1381,8 @@ impl From<SKESK> for Packet {
 /// packet, as the following example demonstrates.
 ///
 /// [the different key formats]: https://tools.ietf.org/html/rfc4880#section-5.5.2
-/// [a proposal for a new key format]: https://www.ietf.org/id/draft-ietf-openpgp-rfc4880bis-09.html#section-5.5.2
-/// [`Key4::generate_ecc`]: key/struct.Key4.html#method.generate_ecc
+/// [a proposal for a new key format]: https://tools.ietf.org/html/draft-ietf-openpgp-rfc4880bis-09.html#section-5.5.2
+/// [`Key4::generate_ecc`]: key::Key4::generate_ecc()
 ///
 ///
 /// ## Examples
@@ -1374,15 +1407,17 @@ impl From<SKESK> for Packet {
 /// (e.g., to decrypt a message, or to generate a signature).
 ///
 /// [password protect keys]: https://tools.ietf.org/html/rfc4880#section-3.7
-/// [`Key::decrypt_secret`]: #method.decrypt_secret
+/// [`Key::decrypt_secret`]: Key::decrypt_secret()
 ///
 /// # A note on equality
 ///
 /// The implementation of `Eq` for `Key` compares the serialized form
-/// of `Key`s.  Notably this includes the secret key material, but
-/// excludes the `KeyParts` and `KeyRole` marker traits.
-/// To exclude the secret key material from the comparison, use
-/// [`Key::public_cmp`] or [`Key::public_eq`].
+/// of `Key`s.  Comparing or serializing values of `Key<PublicParts,
+/// _>` ignore secret key material, whereas the secret key material is
+/// considered and serialized for `Key<SecretParts, _>`, and for
+/// `Key<UnspecifiedParts, _>` if present.  To explicitly exclude the
+/// secret key material from the comparison, use [`Key::public_cmp`]
+/// or [`Key::public_eq`].
 ///
 /// When merging in secret key material from untrusted sources, you
 /// need to be very careful: secret key material is not
@@ -1395,8 +1430,8 @@ impl From<SKESK> for Packet {
 /// secret key material is usually encrypted, this can't always be
 /// done automatically.
 ///
-/// [`Key::public_cmp`]: #method.public_cmp
-/// [`Key::public_eq`]: #method.public_eq
+/// [`Key::public_cmp`]: Key::public_cmp()
+/// [`Key::public_eq`]: Key::public_eq()
 ///
 /// Compare:
 ///
@@ -1404,26 +1439,35 @@ impl From<SKESK> for Packet {
 /// use sequoia_openpgp as openpgp;
 /// use openpgp::cert::prelude::*;
 /// use openpgp::packet::prelude::*;
+/// use openpgp::packet::key::*;
 ///
 /// # fn main() -> openpgp::Result<()> {
 /// // Generate a new certificate.  It has secret key material.
 /// let (cert, _) = CertBuilder::new()
 ///     .generate()?;
 ///
-/// let sk = cert.primary_key().key();
+/// let sk: &Key<PublicParts, _> = cert.primary_key().key();
 /// assert!(sk.has_secret());
 ///
 /// // Strip the secret key material.
 /// let cert = cert.clone().strip_secret_key_material();
-/// let pk = cert.primary_key().key();
+/// let pk: &Key<PublicParts, _> = cert.primary_key().key();
 /// assert!(! pk.has_secret());
 ///
-/// // Eq compares both the public and the secret bits, so it
-/// // considers pk and sk to be different.
+/// // Eq on Key<PublicParts, _> compares only the public bits, so it
+/// // considers pk and sk to be equal.
+/// assert_eq!(pk, sk);
+///
+/// // Convert to Key<UnspecifiedParts, _>.
+/// let sk: &Key<UnspecifiedParts, _> = sk.parts_as_unspecified();
+/// let pk: &Key<UnspecifiedParts, _> = pk.parts_as_unspecified();
+///
+/// // Eq on Key<UnspecifiedParts, _> compares both the public and the
+/// // secret bits, so it considers pk and sk to be different.
 /// assert_ne!(pk, sk);
 ///
-/// // Key::public_eq only compares the public bits, so it considers
-/// // them to be equal.
+/// // In any case, Key::public_eq only compares the public bits,
+/// // so it considers them to be equal.
 /// assert!(Key::public_eq(pk, sk));
 /// # Ok(())
 /// # }
@@ -1485,28 +1529,28 @@ impl<P: key::KeyParts, R: key::KeyRole> Key<P, R> {
 impl From<Key<key::PublicParts, key::PrimaryRole>> for Packet {
     /// Convert the `Key` struct to a `Packet`.
     fn from(k: Key<key::PublicParts, key::PrimaryRole>) -> Self {
-        Packet::PublicKey(k.into())
+        Packet::PublicKey(k)
     }
 }
 
 impl From<Key<key::PublicParts, key::SubordinateRole>> for Packet {
     /// Convert the `Key` struct to a `Packet`.
     fn from(k: Key<key::PublicParts, key::SubordinateRole>) -> Self {
-        Packet::PublicSubkey(k.into())
+        Packet::PublicSubkey(k)
     }
 }
 
 impl From<Key<key::SecretParts, key::PrimaryRole>> for Packet {
     /// Convert the `Key` struct to a `Packet`.
     fn from(k: Key<key::SecretParts, key::PrimaryRole>) -> Self {
-        Packet::SecretKey(k.into())
+        Packet::SecretKey(k)
     }
 }
 
 impl From<Key<key::SecretParts, key::SubordinateRole>> for Packet {
     /// Convert the `Key` struct to a `Packet`.
     fn from(k: Key<key::SecretParts, key::SubordinateRole>) -> Self {
-        Packet::SecretSubkey(k.into())
+        Packet::SecretSubkey(k)
     }
 }
 
@@ -1517,7 +1561,7 @@ impl<R: key::KeyRole> Key<key::SecretParts, R> {
     /// If the `Key` is password protected, you first need to decrypt
     /// it using [`Key::decrypt_secret`].
     ///
-    /// [`Key::decrypt_secret`]: #method.decrypt_secret
+    /// [`Key::decrypt_secret`]: Key::decrypt_secret()
     ///
     /// # Errors
     ///
@@ -1636,7 +1680,9 @@ impl<R: key::KeyRole> Key<key::SecretParts, R> {
     ///
     /// # Examples
     ///
-    /// Encrypt the primary key:
+    /// This example demonstrates how to encrypt the secret key
+    /// material of every key in a certificate.  Decryption can be
+    /// done the same way with [`Key::decrypt_secret`].
     ///
     /// ```rust
     /// use sequoia_openpgp as openpgp;
@@ -1651,19 +1697,31 @@ impl<R: key::KeyRole> Key<key::SecretParts, R> {
     ///     CertBuilder::general_purpose(None,
     ///                                  Some("Alice Lovelace <alice@example.org>"))
     ///         .generate()?;
-    /// let key = cert.primary_key().key().clone().parts_into_secret()?;
-    /// assert!(key.has_unencrypted_secret());
     ///
-    /// // Encrypt the key's secret key material.
-    /// let key = key.encrypt_secret(&"1234".into())?;
-    /// assert!(! key.has_unencrypted_secret());
+    /// // Encrypt every key.
+    /// let mut encrypted_keys: Vec<Packet> = Vec::new();
+    /// for ka in cert.keys().secret() {
+    ///     assert!(ka.has_unencrypted_secret());
     ///
-    /// // Merge it into the certificate.  Note: `Cert::insert_packets`
+    ///     // Encrypt the key's secret key material.
+    ///     let key = ka.key().clone().encrypt_secret(&"1234".into())?;
+    ///     assert!(! key.has_unencrypted_secret());
+    ///
+    ///     // We cannot merge it right now, because `cert` is borrowed.
+    ///     encrypted_keys.push(if ka.primary() {
+    ///         key.role_into_primary().into()
+    ///     } else {
+    ///         key.role_into_subordinate().into()
+    ///     });
+    /// }
+    ///
+    /// // Merge the keys into the certificate.  Note: `Cert::insert_packets`
     /// // prefers added versions of keys.  So, the encrypted version
     /// // will override the decrypted version.
-    /// let cert = cert.insert_packets(Packet::from(key))?;
+    /// let cert = cert.insert_packets(encrypted_keys)?;
     ///
-    /// // Now the primary key's secret key material is encrypted.
+    /// // Now the every key's secret key material is encrypted.  We'll
+    /// // demonstrate this using the primary key:
     /// let key = cert.primary_key().key().parts_as_secret()?;
     /// assert!(! key.has_unencrypted_secret());
     ///
@@ -1697,8 +1755,6 @@ impl<R: key::KeyRole> Key4<key::SecretParts, R> {
     ///
     /// Fails if the secret key is encrypted.  You can use
     /// [`Key::decrypt_secret`] to decrypt a key.
-    ///
-    /// [`Key::decrypt_secret`]: ../enum.Key.html#method.decrypt_secret
     pub fn into_keypair(self) -> Result<KeyPair> {
         let (key, secret) = self.take_secret();
         let secret = match secret {
@@ -1807,8 +1863,8 @@ impl<P: key::KeyParts, R: key::KeyRole> DerefMut for Key<P, R> {
 /// using the [`PacketParser`].
 ///
 /// [Section 5.13 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-5.13
-/// [streaming serializer]: ../serialize/stream/index.html
-/// [`PacketParser`]: ../parse/index.html
+/// [streaming serializer]: crate::serialize::stream
+/// [`PacketParser`]: crate::parse::PacketParser
 #[derive(PartialEq, Eq, Hash, Clone, Debug)]
 pub enum SEIP {
     /// SEIP packet version 1.
@@ -1866,10 +1922,10 @@ impl DerefMut for SEIP {
 /// using the [streaming serializer], or parsing an encrypted message
 /// using the [`PacketParser`].
 ///
-/// [streaming serializer]: ../serialize/stream/index.html
-/// [`PacketParser`]: ../parse/index.html
+/// [streaming serializer]: crate::serialize::stream
+/// [`PacketParser`]: crate::parse::PacketParser
 ///
-/// This feature is [experimental](../index.html#experimental-features).
+/// This feature is [experimental](super#experimental-features).
 #[non_exhaustive]
 #[derive(PartialEq, Eq, Hash, Clone, Debug)]
 pub enum AED {
