@@ -30,15 +30,15 @@ impl SKESK {
         -> Result<(SymmetricAlgorithm, SessionKey)>
     {
         match self {
-            &SKESK::V4(ref s) => s.decrypt(password),
-            &SKESK::V5(ref s) => s.decrypt(password),
+            SKESK::V4(ref s) => s.decrypt(password),
+            SKESK::V5(ref s) => s.decrypt(password),
         }
     }
 }
 
 #[cfg(test)]
 impl Arbitrary for SKESK {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
+    fn arbitrary(g: &mut Gen) -> Self {
         if bool::arbitrary(g) {
             SKESK::V4(SKESK4::arbitrary(g))
         } else {
@@ -154,9 +154,9 @@ impl SKESK4 {
     /// least as strong as the `payload_algo` as not to weaken the
     /// security of the payload encryption.
     ///
-    ///   [`SymmetricAlgorithm`]: ../../types/enum.SymmetricAlgorithm.html
-    ///   [`SEIP`]: ../enum.SEIP.html
-    ///   [`AED`]: ../enum.AED.html
+    ///   [`SymmetricAlgorithm`]: crate::types::SymmetricAlgorithm
+    ///   [`SEIP`]: super::SEIP
+    ///   [`AED`]: super::AED
     pub fn with_password(payload_algo: SymmetricAlgorithm,
                          esk_algo: SymmetricAlgorithm,
                          s2k: S2K,
@@ -177,7 +177,7 @@ impl SKESK4 {
         // We need to prefix the cipher specifier to the session key.
         let mut psk: SessionKey = vec![0; 1 + session_key.len()].into();
         psk[0] = payload_algo.into();
-        psk[1..].copy_from_slice(&session_key);
+        psk[1..].copy_from_slice(session_key);
         let mut esk = vec![0u8; psk.len()];
 
         for (pt, ct) in psk[..].chunks(block_size)
@@ -215,7 +215,7 @@ impl SKESK4 {
     /// but stored in the packet.  If the packet is serialized again,
     /// it is written out.
     ///
-    ///   [`S2K`]: ../../crypto/enum.S2K.html
+    ///   [`S2K`]: super::super::crypto::S2K
     pub fn esk(&self) -> Result<Option<&[u8]>> {
         self.esk.as_ref()
             .map(|esko| esko.as_ref().map(|esk| &esk[..]))
@@ -252,14 +252,14 @@ impl SKESK4 {
     {
         let key = self.s2k.derive_key(password, self.sym_algo.key_size()?)?;
 
-        if let Some(ref esk) = self.esk()? {
+        if let Some(esk) = self.esk()? {
             // Use the derived key to decrypt the ESK. Unlike SEP &
             // SEIP we have to use plain CFB here.
             let blk_sz = self.sym_algo.block_size()?;
             let iv = vec![0u8; blk_sz];
             let mut dec  = self.sym_algo.make_decrypt_cfb(&key[..], iv)?;
             let mut plain: SessionKey = vec![0u8; esk.len()].into();
-            let cipher = &esk[..];
+            let cipher = esk;
 
             for (pl, ct)
                 in plain[..].chunks_mut(blk_sz).zip(cipher.chunks(blk_sz))
@@ -299,7 +299,7 @@ impl From<SKESK4> for Packet {
 
 #[cfg(test)]
 impl Arbitrary for SKESK4 {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
+    fn arbitrary(g: &mut Gen) -> Self {
         SKESK4::new(SymmetricAlgorithm::arbitrary(g),
                     S2K::arbitrary(g),
                     Option::<Vec<u8>>::arbitrary(g).map(|v| v.into()))
@@ -315,7 +315,7 @@ impl Arbitrary for SKESK4 {
 ///
 /// [Section 5.3 of RFC 4880]: https://tools.ietf.org/html/draft-ietf-openpgp-rfc4880bis-05#section-5.3
 ///
-/// This feature is [experimental](../../index.html#experimental-features).
+/// This feature is [experimental](super::super#experimental-features).
 #[derive(Clone, Debug)]
 pub struct SKESK5 {
     /// Common fields.
@@ -448,9 +448,9 @@ impl SKESK5 {
     /// least as strong as the `payload_algo` as not to weaken the
     /// security of the payload encryption.
     ///
-    ///   [`SymmetricAlgorithm`]: ../../types/enum.SymmetricAlgorithm.html
-    ///   [`SEIP`]: ../enum.SEIP.html
-    ///   [`AED`]: ../enum.AED.html
+    ///   [`SymmetricAlgorithm`]: crate::types::SymmetricAlgorithm
+    ///   [`SEIP`]: super::SEIP
+    ///   [`AED`]: super::AED
     pub fn with_password(payload_algo: SymmetricAlgorithm,
                          esk_algo: SymmetricAlgorithm,
                          esk_aead: AEADAlgorithm, s2k: S2K,
@@ -474,7 +474,7 @@ impl SKESK5 {
 
         // We need to prefix the cipher specifier to the session key.
         let mut esk = vec![0u8; session_key.len()];
-        ctx.encrypt(&mut esk, &session_key);
+        ctx.encrypt(&mut esk, session_key);
 
         // Digest.
         let mut digest = vec![0u8; esk_aead.digest_size()?];
@@ -500,10 +500,10 @@ impl SKESK5 {
         let key = self.s2k().derive_key(password,
                                         self.symmetric_algo().key_size()?)?;
 
-        if let Some(ref esk) = self.esk()? {
+        if let Some(esk) = self.esk()? {
             // Use the derived key to decrypt the ESK.
             let mut cipher = self.aead_algo.context(
-                self.symmetric_algo(), &key, &self.aead_iv()?, CipherOp::Decrypt)?;
+                self.symmetric_algo(), &key, self.aead_iv()?, CipherOp::Decrypt)?;
 
             let ad = [0xc3, 5 /* Version.  */, self.symmetric_algo().into(),
                       self.aead_algo.into()];
@@ -512,7 +512,7 @@ impl SKESK5 {
             let mut digest = vec![0; self.aead_algo.digest_size()?];
             cipher.decrypt(&mut plain, esk);
             cipher.digest(&mut digest);
-            if &digest[..] == &self.aead_digest[..] {
+            if digest[..] == self.aead_digest[..] {
                 Ok((SymmetricAlgorithm::Unencrypted, plain))
             } else {
                 Err(Error::ManipulatedMessage.into())
@@ -541,7 +541,7 @@ impl SKESK5 {
     /// but stored in the packet.  If the packet is serialized again,
     /// it is written out.
     ///
-    ///   [`S2K`]: ../../crypto/enum.S2K.html
+    ///   [`S2K`]: super::super::crypto::S2K
     pub fn aead_iv(&self) -> Result<&[u8]> {
         self.aead_iv.as_ref()
             .map(|iv| &iv[..])
@@ -579,7 +579,7 @@ impl From<SKESK5> for Packet {
 
 #[cfg(test)]
 impl Arbitrary for SKESK5 {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
+    fn arbitrary(g: &mut Gen) -> Self {
         let algo = AEADAlgorithm::EAX;  // The only one we dig.
         let mut iv = vec![0u8; algo.iv_size().unwrap()];
         for b in iv.iter_mut() {
