@@ -52,7 +52,7 @@ pub enum Tag {
     MDC,
     /// AEAD Encrypted Data Packet.
     ///
-    /// This feature is [experimental](../index.html#experimental-features).
+    /// This feature is [experimental](crate#experimental-features).
     AED,
     /// Unassigned packets (as of RFC4880).
     Unknown(u8),
@@ -131,27 +131,27 @@ impl From<Tag> for u8 {
 impl From<&Tag> for u8 {
     fn from(t: &Tag) -> u8 {
         match t {
-            &Tag::Reserved => 0,
-            &Tag::PKESK => 1,
-            &Tag::Signature => 2,
-            &Tag::SKESK => 3,
-            &Tag::OnePassSig => 4,
-            &Tag::SecretKey => 5,
-            &Tag::PublicKey => 6,
-            &Tag::SecretSubkey => 7,
-            &Tag::CompressedData => 8,
-            &Tag::SED => 9,
-            &Tag::Marker => 10,
-            &Tag::Literal => 11,
-            &Tag::Trust => 12,
-            &Tag::UserID => 13,
-            &Tag::PublicSubkey => 14,
-            &Tag::UserAttribute => 17,
-            &Tag::SEIP => 18,
-            &Tag::MDC => 19,
-            &Tag::AED => 20,
-            &Tag::Private(x) => x,
-            &Tag::Unknown(x) => x,
+            Tag::Reserved => 0,
+            Tag::PKESK => 1,
+            Tag::Signature => 2,
+            Tag::SKESK => 3,
+            Tag::OnePassSig => 4,
+            Tag::SecretKey => 5,
+            Tag::PublicKey => 6,
+            Tag::SecretSubkey => 7,
+            Tag::CompressedData => 8,
+            Tag::SED => 9,
+            Tag::Marker => 10,
+            Tag::Literal => 11,
+            Tag::Trust => 12,
+            Tag::UserID => 13,
+            Tag::PublicSubkey => 14,
+            Tag::UserAttribute => 17,
+            Tag::SEIP => 18,
+            Tag::MDC => 19,
+            Tag::AED => 20,
+            Tag::Private(x) => *x,
+            Tag::Unknown(x) => *x,
         }
     }
 }
@@ -219,7 +219,7 @@ impl fmt::Display for Tag {
 
 #[cfg(test)]
 impl Arbitrary for Tag {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
+    fn arbitrary(g: &mut Gen) -> Self {
         loop {
             match u8::arbitrary(g) {
                 n @ 0..=63 => break n.into(),
@@ -283,7 +283,7 @@ mod tests {
         fn unknown_private(tag: Tag) -> bool {
             match tag {
                 Tag::Unknown(u) => u > 19 || u == 15 || u == 16,
-                Tag::Private(u) => u >= 60 && u <= 63,
+                Tag::Private(u) => (60..=63).contains(&u),
                 _ => true
             }
         }
@@ -291,8 +291,8 @@ mod tests {
 
     #[test]
     fn parse() {
-        for i in 0..0x100usize {
-            Tag::from(i as u8);
+        for i in 0..u8::MAX {
+            let _ = Tag::from(i);
         }
     }
 }
