@@ -98,7 +98,7 @@ impl Unknown {
     /// compromised, but not completely broken.  For more details,
     /// please refer to the documentation for [HashAlgoSecurity].
     ///
-    ///   [HashAlgoSecurity]: ../policy/enum.HashAlgoSecurity.html
+    ///   [HashAlgoSecurity]: crate::policy::HashAlgoSecurity
     pub fn hash_algo_security(&self) -> HashAlgoSecurity {
         HashAlgoSecurity::CollisionResistance
     }
@@ -135,7 +135,7 @@ impl Unknown {
     /// case.
     pub(crate) // For cert/mod.rs
     fn best_effort_cmp(&self, other: &Unknown) -> Ordering {
-        self.tag.cmp(&other.tag).then_with(|| self.body().cmp(&other.body()))
+        self.tag.cmp(&other.tag).then_with(|| self.body().cmp(other.body()))
     }
 }
 
