@@ -146,8 +146,8 @@ impl Cookie {
                 t!("{:?}: group {} {:?} hashing {} bytes.",
                    hashes_for, i, mode.map(|ctx| ctx.algo()), data.len());
                 match mode {
-                    HashingMode::Binary(h) => h.update(&data),
-                    HashingMode::Text(h) => hash_update_text(h, &data),
+                    HashingMode::Binary(h) => h.update(data),
+                    HashingMode::Text(h) => hash_update_text(h, data),
                 }
             }
         }
@@ -357,7 +357,7 @@ impl<R: BufferedReader<Cookie>>
 /// This can be used to verify detached signatures.  For a more
 /// convenient method, see [`DetachedVerifier`].
 ///
-///  [`DetachedVerifier`]: ../parse/stream/struct.DetachedVerifier.html
+///  [`DetachedVerifier`]: crate::parse::stream::DetachedVerifier
 pub(crate) fn hash_buffered_reader<R>(reader: R,
                                       algos: &[HashingMode<HashAlgorithm>])
     -> Result<Vec<HashingMode<Box<dyn crate::crypto::hash::Digest>>>>
@@ -370,8 +370,7 @@ pub(crate) fn hash_buffered_reader<R>(reader: R,
     reader.drop_eof()?;
 
     let hashes =
-        mem::replace(&mut reader.cookie_mut().sig_group_mut().hashes,
-                     Default::default());
+        mem::take(&mut reader.cookie_mut().sig_group_mut().hashes);
     Ok(hashes)
 }
 
@@ -428,8 +427,7 @@ mod test {
 
             let cookie = reader.cookie_mut();
 
-            let mut hashes = mem::replace(&mut cookie.sig_group_mut().hashes,
-                                          Default::default());
+            let mut hashes = std::mem::take(&mut cookie.sig_group_mut().hashes);
             for mode in hashes.iter_mut() {
                 let hash = mode.as_mut();
                 let algo = hash.algo();
