@@ -40,10 +40,7 @@
 //! that adding or subtracting durations will never overflow or underflow without
 //! notice.
 //!
-//! [`Timestamp::round_down`]: struct.Timestamp.html#method.round_down
-//! [`KeyFlags`]: struct.KeyFlags.html
-//! [`Timestamp`]: struct.Timestamp.html
-//! [`Duration`]: struct.Duration.html
+//! [`Timestamp::round_down`]: Timestamp::round_down()
 
 use std::fmt;
 use std::str::FromStr;
@@ -151,7 +148,10 @@ impl PublicKeyAlgorithm {
                      | DSA
                      | ECDSA
                      | ElGamalEncryptSign
-                     | EdDSA)
+                     | EdDSA
+                     | Private(_)
+                     | Unknown(_)
+            )
         }
     }
 
@@ -174,7 +174,10 @@ impl PublicKeyAlgorithm {
                      | RSAEncrypt
                      | ElGamalEncrypt
                      | ECDH
-                     | ElGamalEncryptSign)
+                     | ElGamalEncryptSign
+                     | Private(_)
+                     | Unknown(_)
+            )
         }
     }
 
@@ -192,14 +195,7 @@ impl PublicKeyAlgorithm {
     /// assert!(!PublicKeyAlgorithm::Private(101).is_supported());
     /// ```
     pub fn is_supported(&self) -> bool {
-        use self::PublicKeyAlgorithm::*;
-        #[allow(deprecated)]
-        match &self {
-            RSAEncryptSign | RSAEncrypt | RSASign | DSA | ECDH | ECDSA | EdDSA
-                => true,
-            ElGamalEncrypt | ElGamalEncryptSign | Private(_) | Unknown(_)
-                => false,
-        }
+        self.is_supported_by_backend()
     }
 }
 
@@ -267,27 +263,20 @@ impl fmt::Display for PublicKeyAlgorithm {
 
 #[cfg(test)]
 impl Arbitrary for PublicKeyAlgorithm {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
+    fn arbitrary(g: &mut Gen) -> Self {
         u8::arbitrary(g).into()
     }
 }
 
 #[cfg(test)]
 impl PublicKeyAlgorithm {
-    pub(crate) fn arbitrary_for_signing<G: Gen>(g: &mut G) -> Self {
-        use rand::Rng;
+    pub(crate) fn arbitrary_for_signing(g: &mut Gen) -> Self {
         use self::PublicKeyAlgorithm::*;
+
         #[allow(deprecated)]
-        let a = match g.gen_range(0, 5) {
-            0 => RSAEncryptSign,
-            1 => RSASign,
-            2 => DSA,
-            3 => ECDSA,
-            4 => EdDSA,
-            _ => unreachable!(),
-        };
+        let a = g.choose(&[RSAEncryptSign, RSASign, DSA, ECDSA, EdDSA]).unwrap();
         assert!(a.for_signing());
-        a
+        *a
     }
 }
 
@@ -391,6 +380,7 @@ const ED25519_OID: &[u8] =
 const CV25519_OID: &[u8] =
     &[0x2B, 0x06, 0x01, 0x04, 0x01, 0x97, 0x55, 0x01, 0x05, 0x01];
 
+#[allow(clippy::len_without_is_empty)]
 impl Curve {
     /// Parses the given OID.
     ///
@@ -430,14 +420,14 @@ impl Curve {
     /// ```
     pub fn oid(&self) -> &[u8] {
         match self {
-            &Curve::NistP256 => NIST_P256_OID,
-            &Curve::NistP384 => NIST_P384_OID,
-            &Curve::NistP521 => NIST_P521_OID,
-            &Curve::BrainpoolP256 => BRAINPOOL_P256_OID,
-            &Curve::BrainpoolP512 => BRAINPOOL_P512_OID,
-            &Curve::Ed25519 => ED25519_OID,
-            &Curve::Cv25519 => CV25519_OID,
-            &Curve::Unknown(ref oid) => oid,
+            Curve::NistP256 => NIST_P256_OID,
+            Curve::NistP384 => NIST_P384_OID,
+            Curve::NistP521 => NIST_P521_OID,
+            Curve::BrainpoolP256 => BRAINPOOL_P256_OID,
+            Curve::BrainpoolP512 => BRAINPOOL_P512_OID,
+            Curve::Ed25519 => ED25519_OID,
+            Curve::Cv25519 => CV25519_OID,
+            Curve::Unknown(ref oid) => oid,
         }
     }
 
@@ -461,14 +451,14 @@ impl Curve {
     /// supported.
     pub fn len(&self) -> Result<usize> {
         match self {
-            &Curve::NistP256 => Ok(256),
-            &Curve::NistP384 => Ok(384),
-            &Curve::NistP521 => Ok(521),
-            &Curve::BrainpoolP256 => Ok(256),
-            &Curve::BrainpoolP512 => Ok(512),
-            &Curve::Ed25519 => Ok(256),
-            &Curve::Cv25519 => Ok(256),
-            &Curve::Unknown(_) =>
+            Curve::NistP256 => Ok(256),
+            Curve::NistP384 => Ok(384),
+            Curve::NistP521 => Ok(521),
+            Curve::BrainpoolP256 => Ok(256),
+            Curve::BrainpoolP512 => Ok(512),
+            Curve::Ed25519 => Ok(256),
+            Curve::Cv25519 => Ok(256),
+            Curve::Unknown(_) =>
                 Err(Error::UnsupportedEllipticCurve(self.clone())
                     .into()),
         }
@@ -482,25 +472,17 @@ impl Curve {
     /// use sequoia_openpgp as openpgp;
     /// use openpgp::types::Curve;
     ///
-    /// assert!(Curve::NistP256.is_supported());
-    /// assert!(Curve::NistP384.is_supported());
     /// assert!(Curve::Ed25519.is_supported());
     /// assert!(!Curve::Unknown(Box::new([0x2B, 0x11])).is_supported());
     /// ```
     pub fn is_supported(&self) -> bool {
-        use self::Curve::*;
-        match &self {
-            NistP256 | NistP384 | NistP521 | Ed25519 | Cv25519
-                => true,
-            BrainpoolP256 | BrainpoolP512 | Unknown(_)
-                => false,
-        }
+        self.is_supported_by_backend()
     }
 }
 
 #[cfg(test)]
 impl Arbitrary for Curve {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
+    fn arbitrary(g: &mut Gen) -> Self {
         match u8::arbitrary(g) % 8 {
             0 => Curve::NistP256,
             1 => Curve::NistP384,
@@ -528,7 +510,7 @@ impl Arbitrary for Curve {
 /// Use [`SymmetricAlgorithm::from`] to translate a numeric value to a
 /// symbolic one.
 ///
-///   [`SymmetricAlgorithm::from`]: https://doc.rust-lang.org/std/convert/trait.From.html
+///   [`SymmetricAlgorithm::from`]: std::convert::From
 ///
 /// Note: This enum cannot be exhaustively matched to allow future
 /// extensions.
@@ -669,7 +651,7 @@ impl fmt::Display for SymmetricAlgorithm {
 
 #[cfg(test)]
 impl Arbitrary for SymmetricAlgorithm {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
+    fn arbitrary(g: &mut Gen) -> Self {
         u8::arbitrary(g).into()
     }
 }
@@ -683,12 +665,12 @@ impl Arbitrary for SymmetricAlgorithm {
 /// Use [`AEADAlgorithm::from`] to translate a numeric value to a
 /// symbolic one.
 ///
-///   [`AEADAlgorithm::from`]: https://doc.rust-lang.org/std/convert/trait.From.html
+///   [`AEADAlgorithm::from`]: std::convert::From
 ///
 /// Note: This enum cannot be exhaustively matched to allow future
 /// extensions.
 ///
-/// This feature is [experimental](../index.html#experimental-features).
+/// This feature is [experimental](super#experimental-features).
 ///
 /// # Examples
 ///
@@ -735,13 +717,7 @@ impl AEADAlgorithm {
     /// assert!(!AEADAlgorithm::OCB.is_supported());
     /// ```
     pub fn is_supported(&self) -> bool {
-        use self::AEADAlgorithm::*;
-        match &self {
-            EAX
-                => true,
-            OCB | Private(_) | Unknown(_)
-                => false,
-        }
+        self.is_supported_by_backend()
     }
 }
 
@@ -784,7 +760,7 @@ impl fmt::Display for AEADAlgorithm {
 
 #[cfg(test)]
 impl Arbitrary for AEADAlgorithm {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
+    fn arbitrary(g: &mut Gen) -> Self {
         u8::arbitrary(g).into()
     }
 }
@@ -926,7 +902,7 @@ impl fmt::Display for CompressionAlgorithm {
 
 #[cfg(test)]
 impl Arbitrary for CompressionAlgorithm {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
+    fn arbitrary(g: &mut Gen) -> Self {
         u8::arbitrary(g).into()
     }
 }
@@ -1096,7 +1072,7 @@ impl HashAlgorithm {
 
 #[cfg(test)]
 impl Arbitrary for HashAlgorithm {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
+    fn arbitrary(g: &mut Gen) -> Self {
         u8::arbitrary(g).into()
     }
 }
@@ -1267,7 +1243,7 @@ impl fmt::Display for SignatureType {
 
 #[cfg(test)]
 impl Arbitrary for SignatureType {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
+    fn arbitrary(g: &mut Gen) -> Self {
         u8::arbitrary(g).into()
     }
 }
@@ -1411,7 +1387,7 @@ impl fmt::Display for ReasonForRevocation {
 
 #[cfg(test)]
 impl Arbitrary for ReasonForRevocation {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
+    fn arbitrary(g: &mut Gen) -> Self {
         u8::arbitrary(g).into()
     }
 }
@@ -1545,7 +1521,7 @@ impl ReasonForRevocation {
 ///
 /// Construct a new [`Message`] containing one text literal packet:
 ///
-/// [`Message`]: struct.Message.html
+/// [`Message`]: crate::Message
 ///
 /// ```rust
 /// use sequoia_openpgp as openpgp;
@@ -1658,7 +1634,7 @@ impl fmt::Display for DataFormat {
 
 #[cfg(test)]
 impl Arbitrary for DataFormat {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
+    fn arbitrary(g: &mut Gen) -> Self {
         u8::arbitrary(g).into()
     }
 }
@@ -1670,7 +1646,7 @@ impl Arbitrary for DataFormat {
 /// Generates a new certificate then checks if the User ID is revoked or not under
 /// the given policy using [`ValidUserIDAmalgamation`]:
 ///
-/// [`ValidUserIDAmalgamation`]: ../cert/amalgamation/type.ValidUserIDAmalgamation.html
+/// [`ValidUserIDAmalgamation`]: crate::cert::amalgamation::ValidUserIDAmalgamation
 ///
 /// ```rust
 /// use sequoia_openpgp as openpgp;
@@ -1729,7 +1705,7 @@ mod tests {
 
     quickcheck! {
         fn comp_roundtrip(comp: CompressionAlgorithm) -> bool {
-            let val: u8 = comp.clone().into();
+            let val: u8 = comp.into();
             comp == CompressionAlgorithm::from(val)
         }
     }
@@ -1745,7 +1721,7 @@ mod tests {
         fn comp_parse(comp: CompressionAlgorithm) -> bool {
             match comp {
                 CompressionAlgorithm::Unknown(u) => u > 110 || (u > 3 && u < 100),
-                CompressionAlgorithm::Private(u) => u >= 100 && u <= 110,
+                CompressionAlgorithm::Private(u) => (100..=110).contains(&u),
                 _ => true
             }
         }
@@ -1754,7 +1730,7 @@ mod tests {
 
     quickcheck! {
         fn sym_roundtrip(sym: SymmetricAlgorithm) -> bool {
-            let val: u8 = sym.clone().into();
+            let val: u8 = sym.into();
             sym == SymmetricAlgorithm::from(val)
         }
     }
@@ -1772,7 +1748,7 @@ mod tests {
                 SymmetricAlgorithm::Unknown(u) =>
                     u == 5 || u == 6 || u > 110 || (u > 10 && u < 100),
                 SymmetricAlgorithm::Private(u) =>
-                    u >= 100 && u <= 110,
+                    (100..=110).contains(&u),
                 _ => true
             }
         }
@@ -1781,7 +1757,7 @@ mod tests {
 
     quickcheck! {
         fn aead_roundtrip(aead: AEADAlgorithm) -> bool {
-            let val: u8 = aead.clone().into();
+            let val: u8 = aead.into();
             aead == AEADAlgorithm::from(val)
         }
     }
@@ -1799,7 +1775,7 @@ mod tests {
                 AEADAlgorithm::Unknown(u) =>
                     u == 0 || u > 110 || (u > 2 && u < 100),
                 AEADAlgorithm::Private(u) =>
-                    u >= 100 && u <= 110,
+                    (100..=110).contains(&u),
                 _ => true
             }
         }
@@ -1808,7 +1784,7 @@ mod tests {
 
     quickcheck! {
         fn pk_roundtrip(pk: PublicKeyAlgorithm) -> bool {
-            let val: u8 = pk.clone().into();
+            let val: u8 = pk.into();
             pk == PublicKeyAlgorithm::from(val)
         }
     }
@@ -1824,9 +1800,9 @@ mod tests {
         fn pk_parse(pk: PublicKeyAlgorithm) -> bool {
             match pk {
                 PublicKeyAlgorithm::Unknown(u) =>
-                    u == 0 || u > 110 || (u >= 4 && u <= 15)
-                    || (u >= 18 && u < 100),
-                PublicKeyAlgorithm::Private(u) => u >= 100 && u <= 110,
+                    u == 0 || u > 110 || (4..=15).contains(&u)
+                    || (18..100).contains(&u),
+                PublicKeyAlgorithm::Private(u) => (100..=110).contains(&u),
                 _ => true
             }
         }
@@ -1842,7 +1818,7 @@ mod tests {
 
     quickcheck! {
         fn signature_type_roundtrip(t: SignatureType) -> bool {
-            let val: u8 = t.clone().into();
+            let val: u8 = t.into();
             t == SignatureType::from(val)
         }
     }
@@ -1857,7 +1833,7 @@ mod tests {
 
     quickcheck! {
         fn hash_roundtrip(hash: HashAlgorithm) -> bool {
-            let val: u8 = hash.clone().into();
+            let val: u8 = hash.into();
             hash == HashAlgorithm::from(val)
         }
     }
@@ -1897,8 +1873,8 @@ mod tests {
         fn hash_parse(hash: HashAlgorithm) -> bool {
             match hash {
                 HashAlgorithm::Unknown(u) => u == 0 || (u > 11 && u < 100) ||
-                    u > 110 || (u >= 4 && u <= 7) || u == 0,
-                HashAlgorithm::Private(u) => u >= 100 && u <= 110,
+                    u > 110 || (4..=7).contains(&u) || u == 0,
+                HashAlgorithm::Private(u) => (100..=110).contains(&u),
                 _ => true
             }
         }
@@ -1906,7 +1882,7 @@ mod tests {
 
     quickcheck! {
         fn rfr_roundtrip(rfr: ReasonForRevocation) -> bool {
-            let val: u8 = rfr.clone().into();
+            let val: u8 = rfr.into();
             rfr == ReasonForRevocation::from(val)
         }
     }
@@ -1926,7 +1902,7 @@ mod tests {
                     || (u > 32 && u < 100)
                     || u > 110,
                 ReasonForRevocation::Private(u) =>
-                    u >= 100 && u <= 110,
+                    (100..=110).contains(&u),
                 _ => true
             }
         }
@@ -1934,7 +1910,7 @@ mod tests {
 
     quickcheck! {
         fn df_roundtrip(df: DataFormat) -> bool {
-            let val: u8 = df.clone().into();
+            let val: u8 = df.into();
             df == DataFormat::from(val)
         }
     }
